/*
 * ProgressWindow.vala
 *
 * Copyright 2015 Tony George <teejee2008@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */


using Gtk;
using Gee;

using TeeJee.Logging;
using TeeJee.FileSystem;
using TeeJee.JSON;
using TeeJee.ProcessManagement;
using TeeJee.GtkHelper;
using TeeJee.System;
using TeeJee.Misc;

public class ProgressWindow : Window {
	private Gtk.Box vbox_main;
	private Gtk.Grid grid_stats;
	private Gtk.Spinner spinner;
	private Gtk.Label lbl_status;
	//private Gtk.Label lbl_status_filename;
	private Gtk.Box hbox_bar;
	private Gtk.DrawingArea drawing_area;

	private Gtk.Label lbl_file_count_value;
	private Gtk.Label lbl_data_value;
	private Gtk.Label lbl_processed_value;
	private Gtk.Label lbl_compressed_value;
	private Gtk.Label lbl_ratio_value;
	private Gtk.Label lbl_elapsed_value;
	private Gtk.Label lbl_remaining_value;
	private Gtk.Label lbl_speed_value;
	private uint timerID = 0;
	private double progress_prev;

	//actions
	private Gtk.Box hbox_actions;
	private Gtk.Button btn_background;
	private Gtk.Button btn_pause;
	private Gtk.Button btn_stop;
	private Gtk.Button btn_finish;

	//window
	private int def_width = 460;
	private uint tmr_init = 0;
	private uint tmr_task = 0;
	private bool task_is_running = false;

	public ProgressWindow() {
		init_window();
	}

	public ProgressWindow.with_parent(Window parent) {
		set_transient_for(parent);
		set_modal(true);
		init_window();
	}

	private void init_window () {
		title = AppName + " v" + AppVersion;
		window_position = WindowPosition.CENTER;
		resizable = false;
		deletable = false;
		icon = get_app_icon(16);

		//vbox_main
		vbox_main = new Box (Orientation.VERTICAL, 6);
		vbox_main.margin = 6;
		add (vbox_main);

		init_labels();
		init_progress_bar();
		init_command_buttons();

		show_all();

		gtk_do_events();

		tmr_init = Timeout.add(500, init_delayed);
	}

	private bool init_delayed() {

		/* any actions that need to run after window has been displayed */

		if (tmr_init > 0) {
			Source.remove(tmr_init);
			tmr_init = 0;
		}

		start();
		
		return false;
	}

	private void init_labels() {
		//grid_stats
		grid_stats = new Grid();
		grid_stats.set_column_spacing (6);
		grid_stats.set_row_spacing (6);
		grid_stats.column_homogeneous = true;
		grid_stats.hexpand = true;
		vbox_main.add(grid_stats);

		int row = -1;

		//lbl_file_count -----------------------------------------
		var lbl_file_count = new Gtk.Label(_("Files:"));
		lbl_file_count.xalign = (float) 0.0;
		grid_stats.attach(lbl_file_count, 0, ++row, 1, 1);

		//lbl_file_count_value
		lbl_file_count_value = new Gtk.Label(_("???"));
		lbl_file_count_value.xalign = (float) 1.0;
		grid_stats.attach(lbl_file_count_value, 1, row, 1, 1);

		//lbl_elapsed -----------------------------------------
		var lbl_elapsed = new Gtk.Label(_("Elapsed:"));
		lbl_elapsed.xalign = (float) 0.0;
		grid_stats.attach(lbl_elapsed, 0, ++row, 1, 1);

		//lbl_elapsed_value
		lbl_elapsed_value = new Gtk.Label(_("???"));
		lbl_elapsed_value.xalign = (float) 1.0;
		grid_stats.attach(lbl_elapsed_value, 1, row, 1, 1);

		//lbl_remaining -----------------------------------------
		var lbl_remaining = new Gtk.Label(_("Remaining:"));
		lbl_remaining.xalign = (float) 0.0;
		grid_stats.attach(lbl_remaining, 0, ++row, 1, 1);

		//lbl_remaining_value
		lbl_remaining_value = new Gtk.Label(_("???"));
		lbl_remaining_value.xalign = (float) 1.0;
		grid_stats.attach(lbl_remaining_value, 1, row, 1, 1);

		//lbl_speed -----------------------------------------
		var lbl_speed = new Gtk.Label(_("Speed:"));
		lbl_speed.xalign = (float) 0.0;
		grid_stats.attach(lbl_speed, 0, ++row, 1, 1);

		//lbl_speed_value
		lbl_speed_value = new Gtk.Label(_("???"));
		lbl_speed_value.xalign = (float) 1.0;
		grid_stats.attach(lbl_speed_value, 1, row, 1, 1);

		row = -1;

		//lbl_data -------------------------------------------------
		var lbl_data = new Gtk.Label(_("Data:"));
		lbl_data.xalign = (float) 0.0;
		lbl_data.margin_left = 12;
		grid_stats.attach(lbl_data, 2, ++row, 1, 1);

		//lbl_data_value
		lbl_data_value = new Gtk.Label(_("???"));
		lbl_data_value.xalign = (float) 1.0;
		grid_stats.attach(lbl_data_value, 3, row, 1, 1);

		//lbl_processed ------------------------------------------
		var lbl_processed = new Gtk.Label(_("Processed:"));
		lbl_processed.xalign = (float) 0.0;
		lbl_processed.margin_left = 12;
		grid_stats.attach(lbl_processed, 2, ++row, 1, 1);

		//lbl_processed_value
		lbl_processed_value = new Gtk.Label(_("???"));
		lbl_processed_value.xalign = (float) 1.0;
		grid_stats.attach(lbl_processed_value, 3, row, 1, 1);

		//lbl_compressed -----------------------------------------
		var lbl_compressed = new Gtk.Label(_("Compressed:"));
		lbl_compressed.xalign = (float) 0.0;
		lbl_compressed.margin_left = 12;
		grid_stats.attach(lbl_compressed, 2, ++row, 1, 1);

		//lbl_compressed_value
		lbl_compressed_value = new Gtk.Label(_("???"));
		lbl_compressed_value.xalign = (float) 1.0;
		grid_stats.attach(lbl_compressed_value, 3, row, 1, 1);

		//lbl_ratio -----------------------------------------
		var lbl_ratio = new Gtk.Label(_("Ratio:"));
		lbl_ratio.xalign = (float) 0.0;
		lbl_ratio.margin_left = 12;
		grid_stats.attach(lbl_ratio, 2, ++row, 1, 1);

		//lbl_ratio_value
		lbl_ratio_value = new Gtk.Label(_("???"));
		lbl_ratio_value.xalign = (float) 1.0;
		grid_stats.attach(lbl_ratio_value, 3, row, 1, 1);

		//hbox_status_line ---------------------------------------

		var hbox_status_line = new Box (Orientation.HORIZONTAL, 6);
		hbox_status_line.margin_top = 12;
		vbox_main.add (hbox_status_line);

		spinner = new Gtk.Spinner();
		hbox_status_line.add(spinner);

		//lbl_status
		lbl_status = new Gtk.Label("");
		lbl_status.xalign = (float) 0.0;
		lbl_status.ellipsize = Pango.EllipsizeMode.MIDDLE;
		lbl_status.max_width_chars = 50;
		hbox_status_line.add(lbl_status);
	}

	private void init_progress_bar() {
		drawing_area = new Gtk.DrawingArea();
		drawing_area.set_size_request(-1, 40);
		drawing_area.hexpand = true;
		
		var sw_progress = new ScrolledWindow(null, null);
		sw_progress.set_shadow_type (ShadowType.ETCHED_IN);
		sw_progress.hscrollbar_policy = PolicyType.NEVER;
		sw_progress.vscrollbar_policy = PolicyType.NEVER;
		sw_progress.expand = true;
		sw_progress.add (drawing_area);

		hbox_bar = new Box (Orientation.HORIZONTAL, 6);
		vbox_main.add (hbox_bar);
		hbox_bar.add(sw_progress);
		
		drawing_area.draw.connect ((context) => {
			weak Gtk.StyleContext style_context = drawing_area.get_style_context ();

			var color_default = style_context.get_color (0);

			//https://www.google.co.in/design/spec/style/color.html#color-color-palette

			var color_blue_100 = Gdk.RGBA();
			color_blue_100.parse("#BBDEFB");
			color_blue_100.alpha = 1.0;

			var color_blue_200 = Gdk.RGBA();
			color_blue_200.parse("#90CAF9");
			color_blue_200.alpha = 1.0;

			var color_blue_300 = Gdk.RGBA();
			color_blue_300.parse("#64B5F6");
			color_blue_300.alpha = 1.0;

			var color_white = Gdk.RGBA();
			color_white.parse("white");
			color_white.alpha = 1.0;

			var color_black = Gdk.RGBA();
			color_black.parse("black");
			color_black.alpha = 1.0;

			var color_grey_700 = Gdk.RGBA();
			color_grey_700.parse("#616161");
			color_grey_700.alpha = 1.0;

			var color_grey_800 = Gdk.RGBA();
			color_grey_800.parse("#424242");
			color_grey_800.alpha = 1.0;

			var color_grey_D8 = Gdk.RGBA();
			color_grey_D8.parse("#D8D8D8");
			color_grey_D8.alpha = 1.0;

			var color_grey_BD = Gdk.RGBA();
			color_grey_BD.parse("#BDBDBD");
			color_grey_BD.alpha = 1.0;

			var color_grey_A4 = Gdk.RGBA();
			color_grey_A4.parse("#A4A4A4");
			color_grey_A4.alpha = 1.0;

			var color_grey_84 = Gdk.RGBA();
			color_grey_84.parse("#848484");
			color_grey_84.alpha = 1.0;

			var color_red = Gdk.RGBA();
			color_red.parse("red");
			color_red.alpha = 1.0;

			var color_blue = Gdk.RGBA();
			color_blue.parse("blue");
			color_blue.alpha = 1.0;

			var color_progress = color_grey_BD;
			var color_ratio = color_grey_84;
			
			color_default = color_black;

			int w = drawing_area.get_allocated_width();
			int h = drawing_area.get_allocated_height();

			int x = 0;
			int y = 0;

			
			//------ BEGIN CONTEXT -------------------------------------------------
			//Draw lighter bar for processed data
			
			context.set_line_width (1);
			Gdk.cairo_set_source_rgba (context, color_progress);

			if (App.archiver.progress > progress_prev) {
				x = (int)(App.archiver.progress * w);
				progress_prev = App.archiver.progress;
			}
			else {
				x = (int)(progress_prev * w);
			}
			context.rectangle(0, 0, x, h);

			context.fill();
			//------ END CONTEXT ---------------------------------------------------

			if (App.archiver.progress > 0) {
				//------ BEGIN CONTEXT -------------------------------------------------
				//Draw progress % text
				
				context.set_line_width (1);
				context.set_font_size(12);
				Gdk.cairo_set_source_rgba (context, color_grey_700);

				y = (int) (h / 2.0);
				context.move_to (w - 40, y + 3);
				context.show_text("%.0f %%".printf(App.archiver.progress * 100));
				//log_msg("%.0f %%".printf(App.archiver.progress * 100));
				context.stroke();
				//------ END CONTEXT ---------------------------------------------------
			}

			if (App.archiver.action == ArchiveAction.CREATE){
				
				//------ BEGIN CONTEXT -------------------------------------------------
				//Draw darker bar for compressed data
				
				context.set_line_width (1);
				Gdk.cairo_set_source_rgba (context, color_ratio);

				x = (int)((App.archiver.compressed_bytes * 1.0 * w ) / App.archive.size) ;
				context.rectangle(0, 0, x, h);

				context.fill();
				//------ END CONTEXT ---------------------------------------------------
			}
			
			if ((App.archiver.action == ArchiveAction.CREATE)
				&& (App.archiver.compression_ratio > 0)) {
					
				//------ BEGIN CONTEXT -------------------------------------------------
				//Draw compression ratio text
				
				context.set_line_width (1);
				context.set_font_size(12);
				Gdk.cairo_set_source_rgba (context, color_grey_700);

				y = (int) (h / 2.0);

				if (x > (w - 40)) {
					x = 0;
				}
				context.move_to (x + 3, y + 3);
				context.show_text("%.0f %%".printf(App.archiver.compression_ratio));
				context.stroke();
				//------ END CONTEXT ---------------------------------------------------
			}

			//------ BEGIN CONTEXT -------------------------------------------------
			context.set_line_width (1);
			Gdk.cairo_set_source_rgba (context, color_black);

			context.move_to(0, 0);
			context.line_to(w, 0);
			context.line_to(w, h);
			context.line_to(0, h);
			context.line_to(0, 0);

			context.stroke();
			//------ END CONTEXT ---------------------------------------------------

			return true;
		});
	}
	
	private void redraw_progressbar() {
		drawing_area.queue_draw_area(0, 0,
		                             drawing_area.get_allocated_width(),
		                             drawing_area.get_allocated_height());
	}

	private void init_command_buttons() {
		//hbox_actions
		hbox_actions = new Gtk.Box (Gtk.Orientation.HORIZONTAL,6);
		hbox_actions.margin_top = 6;
		hbox_actions.homogeneous = true;
		vbox_main.add(hbox_actions);

		hbox_actions.set_size_request(def_width, -1);
		
		//btn_background ---------------------------------------------------

		btn_background = new Gtk.Button.from_stock ("gtk-execute");
		btn_background.set_tooltip_text (_("Run in background with lower priority"));
		hbox_actions.add(btn_background);

		btn_background.label = _("Background");
		btn_background.always_show_image = true;
		btn_background.image_position = PositionType.LEFT;
		btn_background.image = get_shared_icon("gtk-execute", "gtk-execute.png", 16);
		
		btn_background.clicked.connect(() => {
			App.archiver.background_mode = true;
			App.archiver.set_priority();
			this.iconify();
		});

		//btn_pause ---------------------------------------------------

		btn_pause = new Gtk.Button.from_stock ("gtk-media-pause");
		btn_pause.set_tooltip_text (_("Pause"));
		hbox_actions.add(btn_pause);

		btn_pause.label = _("Pause");
		btn_pause.always_show_image = true;
		btn_pause.image_position = PositionType.LEFT;
		btn_pause.image = get_shared_icon("gtk-media-pause", "gtk-media-pause.png", 16);

		btn_pause.clicked.connect(() => {
			switch (App.archiver.status) {
			case AppStatus.RUNNING:
				App.archiver.pause();
				spinner.stop();
				spinner.visible = false;
				break;
			case AppStatus.PAUSED:
				App.archiver.resume();
				spinner.start();
				spinner.visible = true;
				break;
			}

			switch (App.archiver.status) {
			case AppStatus.RUNNING:
				btn_pause.label = _("Pause");
				btn_pause.set_tooltip_text (_("Pause"));
				btn_pause.image = get_shared_icon("gtk-media-pause", "gtk-media-pause.png", 16);
				break;
			case AppStatus.PAUSED:
				btn_pause.label = _("Resume");
				btn_pause.set_tooltip_text (_("Resume"));
				btn_pause.image = get_shared_icon("gtk-media-start", "gtk-media-start.png", 16);
				break;
			}
		});

		//btn_stop -----------------------------------------------------
		
		btn_stop = new Gtk.Button.from_stock ("gtk-media-stop");
		btn_stop.set_tooltip_text (_("Stop"));
		hbox_actions.add(btn_stop);

		btn_stop.label = _("Stop");
		btn_stop.always_show_image = true;
		btn_stop.image_position = PositionType.LEFT;
		btn_stop.image = get_shared_icon("gtk-media-stop", "gtk-media-stop.png", 16);

		btn_stop.clicked.connect(() => {
			App.archiver.stop(AppStatus.CANCELLED);
			gtk_do_events();
		});

		//btn_finish ---------------------------------------------------

		btn_finish = new Gtk.Button.from_stock ("gtk-ok");
		btn_finish.set_tooltip_text (_("Close this window"));
		btn_finish.set_size_request(100,30);
		btn_finish.no_show_all = true;
		hbox_actions.add(btn_finish);
		hbox_actions.set_child_packing(btn_finish,false,false,0,PackType.START);
		
		btn_finish.label = _("OK");
		btn_finish.always_show_image = true;
		btn_finish.image_position = PositionType.LEFT;
		btn_finish.image = get_shared_icon("gtk-ok", "gtk-ok.png", 16);

		btn_finish.clicked.connect(() => {
			this.close();
			gtk_do_events();
		});
		
		btn_background.get_style_context().add_class(Gtk.STYLE_CLASS_LINKED);
		btn_pause.get_style_context().add_class(Gtk.STYLE_CLASS_LINKED);
		btn_stop.get_style_context().add_class(Gtk.STYLE_CLASS_LINKED);
	}

	private void start() {
		log_debug("ProgressWindow: start()");
		//App.archive.print_archive_action();
		
		this.present();
		
		spinner.start();
		spinner.visible = true;
		gtk_do_events();
		
		lbl_status.label = "Preparing...";
		progress_prev = 0.0;
		App.archiver.progress = 0.0;

		hbox_actions.no_show_all = false;
		hbox_actions.show_all();

		btn_pause.set_tooltip_text (_("Pause"));
		btn_pause.image = get_shared_icon("gtk-media-pause", "gtk-media-pause.png", 16);

		lbl_file_count_value.label = "%'d".printf(App.archive.file_count_total);
		lbl_data_value.label = format_file_size(App.archive.size);

		gtk_do_events();

		task_is_running = true;
		
		switch(App.archiver.action){
		case ArchiveAction.CREATE:
			App.compress();
			break;
			
		case ArchiveAction.EXTRACT:
			bool ok = true;
			if (App.arg_prompt_outpath && (App.archiver.extraction_path.length == 0)){
				ok = prompt_for_extraction_path();
			}
			
			if (ok){
				App.extract();
			}
			break;
			
		case ArchiveAction.TEST:
			App.test();
			break;
		}
		
		gtk_do_events();

		timerID = Timeout.add (500, update_status);
	}

	private bool update_status() {
		//App.archive.print_archive_action();
		
		switch (App.archiver.status) {
		case AppStatus.PAUSED:
			this.title = "CPU: %0.0f %%".printf(ProcStats.get_cpu_usage());
			break;

		case AppStatus.RUNNING:
			this.title = "CPU: %0.0f %%".printf(ProcStats.get_cpu_usage());

			App.archiver.query_io_stats();

			// status line
			lbl_status.label = App.archiver.stat_status_line;

			// elapsed time
			lbl_elapsed_value.label = App.archiver.stat_time_elapsed;

			// remaining time
			lbl_remaining_value.label =  App.archiver.stat_time_remaining;

			// file count
			lbl_file_count_value.label = App.archiver.stat_file_count;

			// processed size
			lbl_processed_value.label = App.archiver.stat_data_processed;

			// compressed size
			lbl_compressed_value.label = App.archiver.stat_data_compressed;

			// compression ratio
			lbl_ratio_value.label = App.archiver.stat_compression_ratio;

			// speed
			lbl_speed_value.label = App.archiver.stat_speed;
			
			gtk_do_events();

			break;

		case AppStatus.PASSWORD_REQUIRED:
			// remove progress timers
			Source.remove (timerID);
			// hide window
			this.hide();
			// prompt for password
			tmr_task = Timeout.add(200, prompt_for_password_and_restart_task);
			break;
			
		case AppStatus.FINISHED:
			task_finish();
			break;
			
		case AppStatus.CANCELLED:
			this.close();
			break;
		}

		redraw_progressbar();

		return true;
	}
	
	private void task_finish(){

		// remove progress timers
		Source.remove (timerID);

		this.hide();

		// check status ---------------------------------------

		var status = App.archiver.exit_code;

		log_debug("status code: %d".printf(status));
		
		if (status == 0){ // valid archive, success
			task_is_running = false;

			switch (App.archiver.action){
			case ArchiveAction.CREATE:
				break;
			case ArchiveAction.EXTRACT:
				if (App.arg_same_path){
					App.merge_nested_extraction_directories();
				}
				break;
			case ArchiveAction.TEST:
				gtk_messagebox("","Archive is OK",this,false);
				break;
			}
		}
		else if ((App.archiver.action == ArchiveAction.LIST)
			&& (App.archive.archive_type.length == 0)
			&& (App.archive.children.keys.size == 0)){

			// invalid archive, error
			var msg = _("File is not an archive or format is unsupported");
			gtk_messagebox("Unknown Format", msg, this, true);
			task_is_running = false;
		}
		else{
			// valid archive, error
			switch (App.archiver.action){
			case ArchiveAction.CREATE:
			case ArchiveAction.TEST:
			case ArchiveAction.EXTRACT:
				var msg = _("There were errors while processing the archive.") + "\n\n";
				msg += App.archiver.error_msg.strip();
				gtk_messagebox("Error", msg ,this,true);
				break;
			}
		}

		this.close();
	}

	private bool prompt_for_password_and_restart_task(){
		if (tmr_task > 0) {
			Source.remove(tmr_task);
			tmr_task = 0;
		}

		this.hide();

		string txt = _("Encrypted Archive");
		string msg = _("Enter password");
		App.archive.password = PasswordDialog.prompt_user(this, false, txt, msg);

		if (App.archive.password.length > 0){
			// delete any directory created on previous run
			// this prevents a duplicate directory being created for --extract-same
			if (App.arg_same_path){
				dir_delete(App.archiver.extraction_path);
			}
			
			start(); // start again
		}
		else{
			task_is_running = false; // password was not provided
			log_msg(_("Cancelled by user"));
			this.close();
		}
		
		return true;
	}

	private bool prompt_for_extraction_path(){
		bool ok = false;
		
		this.hide();
		
		//chooser
		var chooser = new Gtk.FileChooserDialog(
			_("Select Extraction Location"),
			this,
			FileChooserAction.SELECT_FOLDER,
			"_Cancel",
			Gtk.ResponseType.CANCEL,
			"_Open",
			Gtk.ResponseType.ACCEPT
		);

		chooser.select_multiple = false;
		//chooser.set_filename(archive_location);

		if (App.last_output_dir.length > 0){
			chooser.set_current_folder(App.last_output_dir);
		}
		else{
			chooser.set_current_folder(App.user_home);
		}
		
		if (chooser.run() == Gtk.ResponseType.ACCEPT) {
			App.archiver.extraction_path = chooser.get_filename();
			App.last_output_dir = App.archiver.extraction_path;
			ok = true;
		}
		else{
			log_msg(_("Cancelled by user"));
			ok = false;
		}
		
		chooser.close();
		gtk_do_events();

		if (ok){
			this.present();
		}
		else{
			this.close();
		}

		return ok;
	}

}


