/*
 * PropertiesWindow.vala
 *
 * Copyright 2015 Tony George <teejee2008@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */


using Gtk;
using Gee;

using TeeJee.Logging;
using TeeJee.FileSystem;
using TeeJee.JSON;
using TeeJee.ProcessManagement;
using TeeJee.GtkHelper;
using TeeJee.System;
using TeeJee.Misc;

public class PropertiesWindow : Gtk.Dialog {
	private Gtk.Box vbox_main;
	private Gtk.DrawingArea area;

	public PropertiesWindow.with_parent(Window parent) {
		set_transient_for(parent);
		set_modal(true);
		init_window();
	}

	public void init_window () {
		title = "Properties";
		window_position = WindowPosition.CENTER;
		resizable = false;
		deletable = true;
		icon = get_app_icon(16);
		
		//vbox_main
		vbox_main = get_content_area () as Gtk.Box;
		vbox_main.margin = 12;
		
		var hbox_content = new Box (Orientation.HORIZONTAL, 6);
		hbox_content.margin_bottom = 12;
		vbox_main.add(hbox_content);

		//ratio bar
		area = new Gtk.DrawingArea();
		area.set_size_request(30, 40);
		area.vexpand = true;
		area.margin_right = 12;
		hbox_content.add(area);

		area.draw.connect(area_draw);
			
		//grid
		var grid = new Grid();
		grid.set_column_spacing (6);
		grid.set_row_spacing (6);
		grid.margin_right = 12;
		//grid.margin_bottom = 12;
		hbox_content.add(grid);

		int row = -1;

		// name ------------------------------------------
		
		var label = new Gtk.Label(_("Name") + ":");
		label.xalign = (float) 1.0;
		label.use_markup = true;
		label.label = "<b>%s</b>".printf(label.label);
		grid.attach(label, 0, ++row, 1, 1);

		// value
		label = new Gtk.Label(file_basename(App.archive.file_path));
		label.xalign = (float) 0.0;
		grid.attach(label, 1, row, 1, 1);

		// location ------------------------------------
		
		label = new Gtk.Label(_("Location") + ":");
		label.xalign = (float) 1.0;
		label.use_markup = true;
		label.label = "<b>%s</b>".printf(label.label);
		grid.attach(label, 0, ++row, 1, 1);

		// value
		label = new Gtk.Label(file_parent(App.archive.file_path));
		label.xalign = (float) 0.0;
		grid.attach(label, 1, row, 1, 1);

		if (App.archive.archive_type.length > 0){
			
			// type ----------------------------------------
			
			label = new Gtk.Label(_("Type") + ":");
			label.xalign = (float) 1.0;
			label.use_markup = true;
			label.label = "<b>%s</b>".printf(label.label);
			grid.attach(label, 0, ++row, 1, 1);

			// value
			label = new Gtk.Label(App.archive.archive_type);
			label.xalign = (float) 0.0;
			grid.attach(label, 1, row, 1, 1);
		}

		if (App.archive.archive_method.length > 0){
			
			// method ------------------------------------------
			
			label = new Gtk.Label(_("Method") + ":");
			label.xalign = (float) 1.0;
			label.use_markup = true;
			label.label = "<b>%s</b>".printf(label.label);
			grid.attach(label, 0, ++row, 1, 1);

			// value
			label = new Gtk.Label(App.archive.archive_method);
			label.xalign = (float) 0.0;
			grid.attach(label, 1, row, 1, 1);
		}
		
		// encrypted ---------------------------------------------
		
		label = new Gtk.Label(_("Encrypted") + ":");
		label.xalign = (float) 1.0;
		label.use_markup = true;
		label.label = "<b>%s</b>".printf(label.label);
		grid.attach(label, 0, ++row, 1, 1);

		// value
		label = new Gtk.Label(App.archive.archive_is_encrypted ? _("Yes") : _("No"));
		label.xalign = (float) 0.0;
		grid.attach(label, 1, row, 1, 1);

		// solid --------------------------------------------
		
		label = new Gtk.Label(_("Solid") + ":");
		label.xalign = (float) 1.0;
		label.use_markup = true;
		label.label = "<b>%s</b>".printf(label.label);
		grid.attach(label, 0, ++row, 1, 1);

		// value
		label = new Gtk.Label(App.archive.archive_is_solid ? _("Yes") : _("No"));
		label.xalign = (float) 0.0;
		grid.attach(label, 1, row, 1, 1);

		if (App.archive.archive_blocks > 0){
			
			// blocks -----------------------------------------
			
			label = new Gtk.Label(_("Blocks") + ":");
			label.xalign = (float) 1.0;
			label.use_markup = true;
			label.label = "<b>%s</b>".printf(label.label);
			grid.attach(label, 0, ++row, 1, 1);

			// value
			label = new Gtk.Label("%'d".printf(App.archive.archive_blocks));
			label.xalign = (float) 0.0;
			grid.attach(label, 1, row, 1, 1);
		}
		
		if (App.archive.file_count_total > 0){
			
			// file_count -----------------------------------------
			
			label = new Gtk.Label(_("Files") + ":");
			label.xalign = (float) 1.0;
			label.use_markup = true;
			label.label = "<b>%s</b>".printf(label.label);
			grid.attach(label, 0, ++row, 1, 1);

			// value
			label = new Gtk.Label("%'ld".printf(App.archive.file_count_total));
			label.xalign = (float) 0.0;
			grid.attach(label, 1, row, 1, 1);
		}
		
		// size_packed ---------------------------------------------
		
		label = new Gtk.Label(_("Packed") + ":");
		label.xalign = (float) 1.0;
		label.use_markup = true;
		label.label = "<b>%s</b>".printf(label.label);
		grid.attach(label, 0, ++row, 1, 1);
		
		// value
		label = new Gtk.Label(
			"%s (%'ld bytes)".printf(
			format_file_size(App.archive.archive_size),
			App.archive.archive_size
			));
		
		label.xalign = (float) 0.0;
		grid.attach(label, 1, row, 1, 1);

		if (App.archive.size > 0){
			
			// size_unpacked -----------------------------------
			
			label = new Gtk.Label(_("Unpacked") + ":");
			label.xalign = (float) 1.0;
			label.use_markup = true;
			label.label = "<b>%s</b>".printf(label.label);
			grid.attach(label, 0, ++row, 1, 1);

			// value
			label = new Gtk.Label(
			"%s (%'ld bytes)".printf(
			format_file_size(App.archive.size),
			App.archive.size
			));
			
			label.xalign = (float) 0.0;
			grid.attach(label, 1, row, 1, 1);
		}

		if (App.archive.archive_header_size > 0){
			
			// size_headers ----------------------------------------
			
			label = new Gtk.Label(_("Headers") + ":");
			label.xalign = (float) 1.0;
			label.use_markup = true;
			label.label = "<b>%s</b>".printf(label.label);
			grid.attach(label, 0, ++row, 1, 1);

			// value
			label = new Gtk.Label("%s (%'ld bytes)".printf(
			format_file_size(App.archive.archive_header_size),
			App.archive.archive_header_size
			));
			
			label.xalign = (float) 0.0;
			grid.attach(label, 1, row, 1, 1);
		}

		// modified date ----------------------------------------
		
		label = new Gtk.Label(_("Modified") + ":");
		label.xalign = (float) 1.0;
		label.use_markup = true;
		label.label = "<b>%s</b>".printf(label.label);
		grid.attach(label, 0, ++row, 1, 1);

		// value
		label = new Gtk.Label(App.archive.archive_modified.format ("%Y-%m-%d %H:%M"));
		label.xalign = (float) 0.0;
		grid.attach(label, 1, row, 1, 1);

		// padding
		label = new Gtk.Label("");
		label.xalign = (float) 0.0;
		label.margin_bottom = 12;
		//grid.attach(label, 0, ++row, 2, 1);
			
		//actions
		//var btn_ok = (Gtk.Button) 
		add_button ("OK", Gtk.ResponseType.OK);
	}

	private bool area_draw(Cairo.Context context) {
		double ratio = (App.archive.archive_size * 1.0) / App.archive.size;

		var color_blue_100 = Gdk.RGBA();
		color_blue_100.parse("#BBDEFB");
		color_blue_100.alpha = 1.0;

		var color_grey_700 = Gdk.RGBA();
		color_grey_700.parse("#616161");
		color_grey_700.alpha = 1.0;

		var color_white = Gdk.RGBA();
		color_white.parse("white");
		color_white.alpha = 1.0;
			
		int w = area.get_allocated_width();
		int h = area.get_allocated_height();
			
		//------ BEGIN CONTEXT -------------------------------------------------
		context.set_line_width (1);
		Gdk.cairo_set_source_rgba (context, color_blue_100);
		context.rectangle(0, 0, w, h);
		context.fill();
		//------ END CONTEXT ---------------------------------------------------

		//------ BEGIN CONTEXT -------------------------------------------------
		context.set_line_width (1);
		Gdk.cairo_set_source_rgba (context, color_grey_700);
		context.rectangle(0, h * (1.0 - ratio), w, (h * ratio));
		context.fill();
		//------ END CONTEXT ---------------------------------------------------
		
		//------ BEGIN CONTEXT -------------------------------------------------
		context.set_line_width (1);
		Gdk.cairo_set_source_rgba (context, color_grey_700);
		context.rectangle(0, 0, w, h);
		context.stroke();
		//------ END CONTEXT ---------------------------------------------------

		//------ BEGIN CONTEXT -------------------------------------------------
		context.set_line_width (1);
		Gdk.cairo_set_source_rgba (context, color_white);
		context.move_to (3, h-6);
		context.show_text("%.0f%%".printf(ratio * 100.00));
		context.stroke();
		//------ END CONTEXT ---------------------------------------------------
		
		return true;
	}
}


