/*
 * SettingsDialog.vala
 *
 * Copyright 2015 Tony George <teejee2008@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */


using Gtk;
using Gee;

using TeeJee.Logging;
using TeeJee.FileSystem;
using TeeJee.JSON;
using TeeJee.ProcessManagement;
using TeeJee.GtkHelper;
using TeeJee.System;
using TeeJee.Misc;

public class SettingsDialog : Gtk.Dialog {
	private Gtk.Notebook notebook;
	private Gtk.TreeView tv_mime;
	private Gtk.ComboBox cmb_app;

	public SettingsDialog.with_parent(Window parent) {
		set_transient_for(parent);
		set_modal(true);
		set_skip_taskbar_hint(true);
		set_skip_pager_hint(true);
		window_position = WindowPosition.CENTER_ON_PARENT;
		deletable = false;
		resizable = false;
		
		icon = get_app_icon(16,".svg");

		title = _("Settings");
		
		// get content area
		var vbox_main = get_content_area();
		vbox_main.spacing = 6;
		vbox_main.margin = 6;
		vbox_main.set_size_request(400,400);

		//notebook
		notebook = new Notebook();
		notebook.tab_pos = PositionType.TOP;
		notebook.show_border = true;
		notebook.scrollable = true;
		//notebook.margin = 6;
		vbox_main.pack_start (notebook, true, true, 0);

		init_ui_tab_general();
		
		init_ui_tab_mimetypes();
		
		// actions -------------------------
		
		// ok
        var button = (Button) add_button ("gtk-ok", Gtk.ResponseType.ACCEPT);
        button.clicked.connect(()=>{
			this.close();
		});

		this.destroy.connect(btn_ok_click);

        show_all();
	}

	private void init_ui_tab_general(){

		var label = new Label (_("General"));

        var vbox = new Box(Orientation.VERTICAL,6);
        vbox.margin = 12;
        notebook.append_page (vbox, label);
        
		// file manager entries
		label = new Label("<b>" + _("File Manager Integration") + "</b>");
		label.set_use_markup(true);
		label.xalign = (float) 0.0;
		label.margin_bottom = 6;
		vbox.add (label);
		
		// add_context_menu
		var chk = new Gtk.CheckButton.with_label(_("Add right-click menu entries") + "*");
		chk.set_tooltip_text(_("Supports Gnome Files, Nautilus, Nemo and Thunar. Changes will take effect on the next login or after restarting the file manager."));
		chk.active = App.add_context_menu;
		chk.margin_left = 6;
		vbox.add(chk);
		var chk_add_context_menu = chk;

		chk.toggled.connect(()=>{
			App.add_context_menu = chk_add_context_menu.active;
		});

		// chk_add_context_submenu
		chk = new Gtk.CheckButton.with_label(_("Display items in a submenu (Nautilus only)"));
		chk.set_tooltip_text("Items will be shown in submenu named 'Crunchy' instead of being added directly to the right-click menu");
		chk.active = App.add_context_submenu;
		chk.margin_left = 6;
		vbox.add(chk);
		var chk_add_context_submenu = chk;
		
		chk.toggled.connect(()=>{
			App.add_context_submenu = chk_add_context_submenu.active;
		});

		// icon size
		label = new Label("<b>" + _("List View") + "</b>");
		label.set_use_markup(true);
		label.xalign = (float) 0.0;
		label.margin_bottom = 6;
		vbox.add (label);
		
		// chk_small_icons_nav_pane
		chk = new Gtk.CheckButton.with_label(_("Use large icons for navigation pane"));
		chk.active = App.use_large_icons_nav_pane;
		chk.margin_left = 6;
		vbox.add(chk);
		var chk_large_icons_nav_pane = chk;
		
		chk.toggled.connect(()=>{
			App.use_large_icons_nav_pane = chk_large_icons_nav_pane.active;
		});

		// chk_small_icons_file_pane
		chk = new Gtk.CheckButton.with_label(_("Use large icons for file view"));
		chk.active = App.use_large_icons_file_pane;
		chk.margin_left = 6;
		vbox.add(chk);
		var chk_large_icons_file_pane = chk;
		
		chk.toggled.connect(()=>{
			App.use_large_icons_file_pane = chk_large_icons_file_pane.active;
		});

		// chk_hide_nav_pane
		chk = new Gtk.CheckButton.with_label(_("Hide navigation pane"));
		chk.active = App.hide_nav_pane;
		chk.margin_left = 6;
		vbox.add(chk);
		var chk_hide_nav_pane = chk;
		
		chk.toggled.connect(()=>{
			App.hide_nav_pane = chk_hide_nav_pane.active;
		});
	}

	private void init_ui_tab_mimetypes(){
		var label = new Label (_("File Assoc"));

        var vbox = new Box(Orientation.VERTICAL,6);
        vbox.margin = 12;
        notebook.append_page (vbox, label);

		var hbox = new Box(Orientation.HORIZONTAL,6);
		vbox.add (hbox);

		// label
		label = new Label(_("Open selected types with:"));
		label.xalign = (float) 0.0;
		label.margin_bottom = 6;
		label.hexpand = true;
		hbox.add(label);

		// cmb_app
		cmb_app = new ComboBox();
		cmb_app.set_tooltip_text(_("Select the application and click Apply"));
		hbox.add (cmb_app);

		// cell_pix
		var cell_pix = new CellRendererPixbuf ();
		cell_pix.stock_size = IconSize.MENU;
		cmb_app.pack_start (cell_pix, false);

		//render icon
		cmb_app.set_cell_data_func (cell_pix, (cell_layout, cell, model, iter) => {
			DesktopApp app;
			model.get (iter, 1, out app, -1);
			(cell as Gtk.CellRendererPixbuf).icon_name = app.icon;
		});
		
		var cell_text = new CellRendererText();
		cmb_app.pack_start(cell_text, false );
		cmb_app.set_cell_data_func (cell_text, (cell_text, cell, model, iter) => {
			string app_name;
			model.get (iter, 0, out app_name, -1);
			(cell as Gtk.CellRendererText).text = app_name;
		});

		cmb_app_refresh();

        // tv_mime
		var tv = new TreeView();
		tv.get_selection().mode = SelectionMode.MULTIPLE;
		tv.headers_clickable = true;
		tv.has_tooltip = true;
		tv.set_rules_hint (true);
		tv_mime = tv;
		
		// sw_mime
		var scroll = new ScrolledWindow(null, null);
		scroll.set_shadow_type (ShadowType.ETCHED_IN);
		scroll.add (tv_mime);
		scroll.vexpand = true;
		vbox.add(scroll);
		//sw_mime = scroll;

		// type -------------------------------------------------

		// column
		var col = new TreeViewColumn();
		col.title = _("File Type");
		//col.clickable = true;
		col.resizable = true;
		col.min_width = 150;
		tv.append_column(col);

		// checkbox
		CellRendererToggle cell_toggle = new CellRendererToggle ();
		cell_toggle.activatable = true;
		col.pack_start (cell_toggle, false);

		// checkbox state
		col.set_cell_data_func (cell_toggle, (cell_layout, cell, model, iter) => {
			bool selected;
			MimeType mime;
			model.get (iter, 0, out selected, 1, out mime,  -1);
			(cell as Gtk.CellRendererToggle).active = selected;
		});

		cell_toggle.toggled.connect((path) => {
			var model = (Gtk.ListStore)tv_mime.model;
			bool selected;
			MimeType mime;
			TreeIter iter;

			model.get_iter_from_string (out iter, path);
			model.get (iter, 0, out selected, 1, out mime,  -1);
			model.set (iter, 0, !selected);
			mime.is_selected = !selected;
		});
		
		// cell_pix
		/*var cell_pix = new CellRendererPixbuf ();
		cell_pix.stock_size = IconSize.BUTTON;
		//col.pack_start (cell_pix, false);

		//render icon
		col.set_cell_data_func (cell_pix, (cell_layout, cell, model, iter) => {
			MimeType mime;
			model.get (iter, 0, out mime, -1);
			(cell as Gtk.CellRendererPixbuf).icon_name = mime.generic_icon;
		});*/
		
		// cell text
		cell_text = new CellRendererText ();
		cell_text.xalign = (float) 0.0;
		cell_text.ellipsize = Pango.EllipsizeMode.END;
		col.pack_start (cell_text, false);

		// render text
		col.set_cell_data_func (cell_text, (cell_layout, cell, model, iter) => {
			MimeType mime;
			bool odd_row;
			model.get (iter, 1, out mime, 2, out odd_row, -1);

			(cell as Gtk.CellRendererText).text = mime.comment;

			if (odd_row){
				(cell as Gtk.CellRendererText).background = "#F4F6F7";
			}
			else{
				(cell as Gtk.CellRendererText).background = "#FFFFFF";
			}
		});

		// application -------------------------------------------------

		// column
		col = new TreeViewColumn();
		col.title = _("Default Application");
		//col.clickable = true;
		col.resizable = true;
		col.min_width = 150;
		tv.append_column(col);

		// cell_pix
		cell_pix = new CellRendererPixbuf ();
		cell_pix.stock_size = IconSize.MENU;
		col.pack_start (cell_pix, false);

		//render icon
		col.set_cell_data_func (cell_pix, (cell_layout, cell, model, iter) => {
			DesktopApp? app;
			bool odd_row;
			model.get (iter, 2, out odd_row, 3, out app, -1);

			if (app != null){
				(cell as Gtk.CellRendererPixbuf).icon_name = app.icon;
			}
			else{
				(cell as Gtk.CellRendererPixbuf).icon_name = "";
			}

			if (odd_row){
				(cell as Gtk.CellRendererPixbuf).cell_background = "#F4F6F7";
			}
			else{
				(cell as Gtk.CellRendererPixbuf).cell_background = "#FFFFFF";
			}
		});
		
		// cell text
		cell_text = new CellRendererText ();
		cell_text.xalign = (float) 0.0;
		cell_text.ellipsize = Pango.EllipsizeMode.END;
		col.pack_start (cell_text, false);

		// render text
		col.set_cell_data_func (cell_text, (cell_layout, cell, model, iter) => {
			DesktopApp app;
			bool odd_row;
			model.get (iter, 2, out odd_row, 3, out app, -1);

			if (app != null){
				(cell as Gtk.CellRendererText).text = app.name;
			}
			else{
				(cell as Gtk.CellRendererText).text = "";
			}

			if (odd_row){
				(cell as Gtk.CellRendererText).background = "#F4F6F7";
			}
			else{
				(cell as Gtk.CellRendererText).background = "#FFFFFF";
			}
		});
		
		// pattern -------------------------------------------------

		// column
		col = new TreeViewColumn();
		col.title = _("Pattern");
		//col.clickable = true;
		col.resizable = true;
		col.min_width = 150;
		//tv.append_column(col);

		// cell text
		cell_text = new CellRendererText ();
		cell_text.xalign = (float) 0.0;
		cell_text.ellipsize = Pango.EllipsizeMode.END;
		col.pack_start (cell_text, false);

		// render text
		col.set_cell_data_func (cell_text, (cell_layout, cell, model, iter) => {
			MimeType mime;
			bool odd_row;
			model.get (iter, 1, out mime, 2, out odd_row, -1);

			(cell as Gtk.CellRendererText).text = mime.pattern;

			if (odd_row){
				(cell as Gtk.CellRendererText).background = "#F4F6F7";
			}
			else{
				(cell as Gtk.CellRendererText).background = "#FFFFFF";
			}
		});

		tv_mime_refresh();


		// actions -------------------------------------------
		
		hbox = new Box(Orientation.HORIZONTAL,6);
		vbox.add (hbox);

		var size_group = new Gtk.SizeGroup(Gtk.SizeGroupMode.HORIZONTAL);

		// select_all
		var button = new Gtk.Button.with_label (_("Select All"));
		hbox.add(button);

		size_group.add_widget(button);
		
		button.clicked.connect(() => {
			foreach(var mime in App.mimetype_list) {
				mime.is_selected = true;
			}
			tv_mime_refresh();
		});

		// select_none
		button = new Gtk.Button.with_label (_("Select None"));
		hbox.add(button);
		
		size_group.add_widget(button);
		
		button.clicked.connect(() => {
			foreach(var mime in App.mimetype_list) {
				mime.is_selected = false;
			}
			tv_mime_refresh();
		});

		// apply
		button = new Gtk.Button.with_label (_("Apply"));
		hbox.add(button);

		size_group.add_widget(button);
		
		button.clicked.connect(update_file_associations);
	}

	private void tv_mime_refresh() {
		log_debug("tv_mime_refresh");

		var model = new Gtk.ListStore(4,
			typeof(bool), typeof(MimeType),
			typeof(bool), typeof(DesktopApp?));
			
		TreeIter iter;

		bool first_iter_found = false;
		TreeIter first_iter;
		bool odd_row = false;
		foreach(MimeType mime in App.mimetype_list) {
			odd_row = !odd_row;

			DesktopApp app = null;
			if (MimeApp.mimeapplist.has_key(mime.type)){
				var mimeapp = MimeApp.mimeapplist[mime.type];
				var desktop_name = mimeapp.applications.split(";")[0];
				if (DesktopApp.applist.has_key(desktop_name)){
					app = DesktopApp.applist[desktop_name];
				}
				else{
					//log_debug("DesktopApp.applist: no_key: %s".printf(desktop_name));
				}
			}
			else{
				//log_debug("MimeApp.mimeapps_local: no_key: %s".printf(mime.type));
			}
			
			//add row
			model.append(out iter);
			model.set (iter, 0, mime.is_selected);
			model.set (iter, 1, mime);
			model.set (iter, 2, odd_row);
			model.set (iter, 3, app);
			
			if (!first_iter_found) {
				first_iter_found = true;
				first_iter = iter;
			}
		}

		tv_mime.set_model(model);
		tv_mime.columns_autosize();
	}

	private void cmb_app_refresh() {
		var store = new Gtk.ListStore(2, typeof (string), typeof (DesktopApp));
		TreeIter iter;

		int index = 0;
		int active = 0;
		foreach(DesktopApp app in DesktopApp.applist.values){
			if (app.mimetypes.contains("application/gzip")
				|| app.mimetypes.contains("application/x-gzip")){

				if (app.desktop_file_name == App.crunchy_app.desktop_file_name){
					active = index;
				}
				
				store.append(out iter);
				store.set (iter, 0, app.name, 1, app, -1);

				index++;
			}
		}
		
		cmb_app.set_model (store);
		cmb_app.active = active;
	}

	private void btn_ok_click(){
		//App.update_cron_jobs();
		App.save_app_config();
		
		App.update_context_menu();
		update_file_associations();
	}

	private void update_file_associations(){
		var app = (DesktopApp) gtk_combobox_get_selected_object(cmb_app,1,App.crunchy_app);

		foreach(var mime in App.mimetype_list) {
			if (mime.is_selected){
				MimeApp.set_default(mime.type, app);
			}
		}

		MimeApp.query_mimeapps(App.user_home);
		tv_mime_refresh();
	}
}


