/*
 * ViewArchiveWindow.vala
 *
 * Copyright 2015 Tony George <teejee2008@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */


using Gtk;
using Gee;

using TeeJee.Logging;
using TeeJee.FileSystem;
using TeeJee.JSON;
using TeeJee.ProcessManagement;
using TeeJee.GtkHelper;
using TeeJee.System;
using TeeJee.Misc;

public class ViewArchiveWindow : Window {
	private Gtk.Box vbox_main;
	private Gtk.Box hbox_path;
	
	//toolbar
	private Gtk.Toolbar toolbar;
	private Gtk.ToolButton btn_open_archive;
	//private Gtk.ToolButton btn_new_archive;
	private Gtk.ToolButton btn_extract_archive;
	private Gtk.ToolButton btn_test_archive;
	//private Gtk.ToolButton btn_create_archive;
	//private Gtk.ToolButton btn_add_files;
	//private Gtk.ToolButton btn_add_folders;
	//private Gtk.ToolButton btn_remove_item;
	private Gtk.ToolButton btn_properties;
	private Gtk.ToolButton btn_settings;
	private Gtk.ToolButton btn_about;
	private Gtk.ToolButton btn_donate;

	//listview
	private Gtk.Paned pane;
	private Gtk.TreeView tv_nav;
	private Gtk.ScrolledWindow sw_nav;
	private Gtk.TreeView tv_files;
	private Gtk.ScrolledWindow sw_files;
	private Gtk.TreeViewColumn col_dir;
	private Gtk.TreeViewColumn col_name;
	private Gtk.TreeViewColumn col_size;
	private Gtk.TreeViewColumn col_compressed;
	private Gtk.TreeViewColumn col_modified;
	private Gtk.TreeViewColumn col_permissions;
	private Gtk.TreeViewColumn col_owner;
	private Gtk.TreeViewColumn col_group;
	private Gtk.TreeViewColumn col_close;
	private Gtk.Image img_nav_toggle;
	private int sort_column_index = 0;
	private bool sort_column_desc = true;
	
	//statusbar
	private Gtk.Box hbox_statusbar;
	private Gtk.Spinner spinner;
	private Gtk.Label lbl_statusbar;
	private Gtk.ProgressBar progressbar;

	//open archives
	private Gee.ArrayList<ArchiveFile> archive_list;
	private Gee.ArrayList<string> selected_archives;
	private ArchiveFile current_task;
	private FileItem current_item;
	
	//window
	private int def_width = 700;
	private int def_height = 500;

	private uint timerID = 0;
	private uint tmr_task = 0;
	private string statusbar_msg = "";
	private bool task_is_running = false;
	private bool open_archive_task_is_running = false;
	
	private SimpleProgressWindow dlg_open;
	
	public ViewArchiveWindow() {
		archive_list = new Gee.ArrayList<ArchiveFile>();
		selected_archives = new Gee.ArrayList<string>();
		init_window();
	}

	//TODO: mark strings for translation

	//init

	public void init_window () {
		title = AppName + " v" + AppVersion;
		window_position = WindowPosition.CENTER;
		set_default_size(def_width, def_height);
		icon = get_app_icon(16);
		resizable = true;

		//vbox_main
		vbox_main = new Box (Orientation.VERTICAL, 0);
		vbox_main.margin = 0;
		add (vbox_main);

		init_toolbar();
		init_tv_nav();
		init_tv_files();
		init_statusbar();

		show_all();

		gtk_do_events();
		
		tv_nav_refresh();
		tv_files_refresh();
		set_statusbar_summary();
		set_window_title();
		set_toolbar_buttons();

		first_run();
		
		switch (App.app_mode) {
		case AppMode.OPEN:
			selected_archives.clear();
			foreach(string file_path in App.arg_files){
				selected_archives.add(file_path);
			}
			open_archives();
			break;
		}
	}

	private void init_toolbar() {
		//toolbar
		toolbar = new Gtk.Toolbar ();
		toolbar.toolbar_style = ToolbarStyle.BOTH_HORIZ;
		toolbar.get_style_context().add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR);
		//toolbar.set_size_request(-1,48);
		vbox_main.add(toolbar);

		//btn_open_archive
		var button = new Gtk.ToolButton.from_stock ("gtk-open");
		button.is_important = true;
		button.label = _("Open");
		button.set_tooltip_text (_("Open an existing archive file"));
		toolbar.add(button);
		btn_open_archive = button;
		
		button.clicked.connect(() => {
			open_archive_select_file();
		});

		//btn_extract_archive
		button = new Gtk.ToolButton.from_stock ("gtk-folder");
		button.is_important = true;
		button.icon_widget = get_shared_icon("extract-archive", "extract-archive.svg", 24);
		button.label = _("Extract");
		button.set_tooltip_text (_("Extract files from this archive"));
		toolbar.add(button);
		btn_extract_archive = button;

		button.clicked.connect(() => {
			App.archiver.action = ArchiveAction.EXTRACT;
			extract_archive_select_path();
		});

		//btn_test_archive
		button = new Gtk.ToolButton.from_stock ("gtk-folder");
		button.is_important = true;
		button.icon_widget = get_shared_icon("test-archive", "test-archive.svg", 24);
		button.label = _("Test");
		button.set_tooltip_text (_("Test archive"));
		toolbar.add(button);
		btn_test_archive = button;

		button.clicked.connect(() => {
			App.archiver.action = ArchiveAction.TEST;
			test_archive();
		});

		/*
		//btn_new_archive
		button = new Gtk.ToolButton.from_stock ("gtk-new");
		button.is_important = true;
		button.label = _("New");
		button.set_tooltip_text (_("Create new archive"));
		//toolbar.add(button);
		btn_new_archive = button;

		button.clicked.connect (() => {
			new_archive();
		});*/

		/*
		//btn_create_archive
		button = new Gtk.ToolButton.from_stock ("gtk-save");
		button.is_important = true;
		button.label = _("Save");
		button.set_tooltip_text (_("Create a new archive"));
		//toolbar.add(button);
		btn_create_archive = button;

		button.clicked.connect (() => {
			create_archive();
		});*/

		/*
		//btn_add_files
		button = new Gtk.ToolButton.from_stock ("gtk-add");
		button.is_important = true;
		button.label = _("Add file(s)");
		button.set_tooltip_text (_("Add files to this archive"));
		//toolbar.add(button);
		btn_add_files = button;

		button.clicked.connect (() => {
			add_files_to_new_archive();
		});*/

		/*
		//btn_add_folders
		button = new Gtk.ToolButton.from_stock ("gtk-directory");
		button.is_important = true;
		button.label = _("Add directory");
		button.set_tooltip_text (_("Add directories to this archive"));
		//toolbar.add(button);
		btn_add_files = button;
		
		button.clicked.connect (() => {
			add_folders_to_new_archive();
		});
		*/

		/*
		//btn_remove_item
		button = new Gtk.ToolButton.from_stock ("gtk-remove");
		button.is_important = true;
		button.label = _("Remove");
		button.set_tooltip_text (_("Remove selected items"));
		//toolbar.add(button);
		btn_remove_item = button;
		
		button.clicked.connect (() => {
			remove_selected_items();
		});
		*/
		
		//btn_properties
		button = new Gtk.ToolButton.from_stock ("gtk-info");
		button.is_important = true;
		button.label = _("Info");
		button.set_tooltip_text (_("Archive Info"));
		button.icon_widget = get_shared_icon("gtk-info", "help-info.svg", 24);
		toolbar.add(button);
		btn_properties = button;
		
		button.clicked.connect (() => {
			var dlg = new PropertiesWindow.with_parent(this);
			dlg.show_all();
			dlg.run();
			dlg.destroy();
		});
		
		//separator
		var separator = new Gtk.SeparatorToolItem();
		separator.set_draw (false);
		separator.set_expand (true);
		toolbar.add (separator);

		//btn_settings
		button = new Gtk.ToolButton.from_stock ("gtk-preferences");
		button.is_important = false;
		button.label = _("Settings");
		button.set_tooltip_text (_("Change Application Settings"));
		//button.icon_widget = get_shared_icon("donate", "donate.svg", 24);
		toolbar.add(button);
		btn_settings = button;
		
		button.clicked.connect(() => {
			var dlg = new SettingsDialog.with_parent(this);
			dlg.destroy.connect(()=>{
				tv_nav_refresh();
				tv_files_refresh();
			});	
		});

		//btn_donate
		button = new Gtk.ToolButton(null, null);
		button.is_important = false;
		button.label = _("Donate");
		button.set_tooltip_text (_("Make a donation"));
		button.icon_widget = get_shared_icon("donate", "donate.svg", 24);
		toolbar.add(button);
		btn_donate = button;
		
		button.clicked.connect(() => {
			var dialog = new DonationWindow();
			dialog.set_transient_for(this);
			dialog.show_all();
			dialog.run();
			dialog.destroy();
		});

		//btn_about
		button = new Gtk.ToolButton.from_stock ("gtk-about");
		button.is_important = false;
		button.label = _("About");
		button.set_tooltip_text (_("Application Info"));
		toolbar.add(button);
		btn_about = button;
		
		button.clicked.connect(() => {
			btn_about_clicked();
		});
	}

	private void init_tv_nav() {
		pane = new Gtk.Paned (Gtk.Orientation.HORIZONTAL);
		pane.position = 200;
		pane.margin = 3;
		vbox_main.add(pane);

		if (App.hide_nav_pane){
			pane.position = 0;
		}

		//tv_nav
		tv_nav = new Gtk.TreeView();
		tv_nav.get_selection().mode = SelectionMode.SINGLE;
		tv_nav.headers_visible = false;
		tv_nav.activate_on_single_click = true;

		//sw_nav
		sw_nav = new Gtk.ScrolledWindow(null, null);
		sw_nav.vexpand = true;
		sw_nav.set_shadow_type (ShadowType.ETCHED_IN);
		sw_nav.margin_right = 3;
		sw_nav.add (tv_nav);

		pane.pack1(sw_nav, true, true); //resize, shrink

		//col_dir
		col_dir = new TreeViewColumn();
		col_dir.expand = true;
		tv_nav.append_column(col_dir);

		var cell_pix = new CellRendererPixbuf ();
		//cell_pix // TODO: Add setting: Large Icons
		//cell_pix.stock_size = IconSize.LARGE_TOOLBAR; // TODO: Add setting: Large Icons
		col_dir.pack_start (cell_pix, false);

		CellRendererText cell_dir_name = new CellRendererText ();
		cell_dir_name.ellipsize = Pango.EllipsizeMode.END;
		col_dir.pack_start (cell_dir_name, false);

		//render icon
		col_dir.set_cell_data_func (cell_pix, (cell_layout, cell, model, iter) => {
			FileItem item;
			model.get (iter, 0, out item, -1);
						
			if (item.parent == null) {
				(cell as Gtk.CellRendererPixbuf).icon_name = "application-x-compress";
			}
			else {
				(cell as Gtk.CellRendererPixbuf).stock_id = "gtk-directory";
			}

			if (App.use_large_icons_nav_pane || (item.parent == null)){
				(cell as Gtk.CellRendererPixbuf).ypad = 6;
				(cell as Gtk.CellRendererPixbuf).stock_size = IconSize.LARGE_TOOLBAR;
			}
			else{
				(cell as Gtk.CellRendererPixbuf).ypad = 0;
				(cell as Gtk.CellRendererPixbuf).stock_size = IconSize.MENU;
			}

			bool odd_row;
			model.get (iter, 1, out odd_row, -1);
			
			if (odd_row){
				(cell as Gtk.CellRendererPixbuf).cell_background = "#F4F6F7";
			}
			else{
				(cell as Gtk.CellRendererPixbuf).cell_background = "#FFFFFF";
			}
		});

		//render text
		col_dir.set_cell_data_func (cell_dir_name, (cell_layout, cell, model, iter) => {
			FileItem item;
			model.get (iter, 0, out item, -1);
			
			(cell as Gtk.CellRendererText).text = item.file_name;

			bool odd_row;
			model.get (iter, 1, out odd_row, -1);
			
			if (odd_row){
				(cell as Gtk.CellRendererText).background = "#F4F6F7";
			}
			else{
				(cell as Gtk.CellRendererText).background = "#FFFFFF";
			}
		});

		//row activated event
		tv_nav.row_activated.connect(tv_nav_row_activated);

		//close icon column
		var col = new TreeViewColumn();
		//col.expand = true;
		tv_nav.append_column(col);
		col_close = col;
		
		cell_pix = new CellRendererPixbuf ();
		cell_pix.ypad = 6; // TODO: Add setting: Large Icons
		cell_pix.xpad = 12;
		cell_pix.stock_size = IconSize.MENU; // TODO: Add setting: Large Icons
		col.pack_start (cell_pix, false);

		//render icon
		col.set_cell_data_func (cell_pix, (cell_layout, cell, model, iter) => {
			FileItem item;
			model.get (iter, 0, out item, -1);
			
			if (item.parent == null) {
				(cell as Gtk.CellRendererPixbuf).icon_name = "gtk-close";
			}
			else {
				(cell as Gtk.CellRendererPixbuf).stock_id = "";
			}

			bool odd_row;
			model.get (iter, 1, out odd_row, -1);
			
			if (odd_row){
				(cell as Gtk.CellRendererPixbuf).cell_background = "#F4F6F7";
			}
			else{
				(cell as Gtk.CellRendererPixbuf).cell_background = "#FFFFFF";
			}
		});
	}

	private void init_tv_files() {
		var vbox = new Box(Orientation.VERTICAL, 3);
		//vbox.margin_left = 3;
		pane.pack2(vbox, true, false); //resize only

		// tv_files
		tv_files = new TreeView();
		tv_files.get_selection().mode = SelectionMode.MULTIPLE;
		tv_files.headers_clickable = true;
		tv_files.show_expanders = false;
		tv_files.rubber_banding = true;
		tv_files.has_tooltip = true;
		tv_files.set_rules_hint (true);

		// sw_files
		sw_files = new ScrolledWindow(null, null);
		sw_files.set_shadow_type (ShadowType.ETCHED_IN);
		sw_files.add (tv_files);
		sw_files.vexpand = true;
		vbox.add(sw_files);
		
		// name ----------------------------------------------

		// column
		var col = new TreeViewColumn();
		col.title = _("Name");
		col.clickable = true;
		col.resizable = true;
		//col.spacing = 1;
		//col.min_width = 200;
		col.expand = true;
		tv_files.append_column(col);
		col_name = col;

		// cell icon
		var cell_pix = new CellRendererPixbuf ();
		col.pack_start (cell_pix, false);

		// cell text
		var cell_text = new CellRendererText ();
		cell_text.ellipsize = Pango.EllipsizeMode.END;
		col.pack_start (cell_text, false);

		// render icon
		col.set_cell_data_func (cell_pix, (cell_layout, cell, model, iter) => {
			FileItem item;
			bool odd_row;
			model.get (iter, 0, out item, 1, out odd_row, -1);

			if (item.is_dummy) {
				(cell as Gtk.CellRendererPixbuf).stock_id = "gtk-directory";
			}
			else if (item.is_archive) {
				(cell as Gtk.CellRendererPixbuf).icon_name = "gnome-mime-application-x-archive";
			}
			else {
				if ((item.icon != null) && item.file_path.has_prefix("/")) {
					(cell as Gtk.CellRendererPixbuf).gicon = item.icon;
				}
				else if (item.is_symlink) {
					(cell as Gtk.CellRendererPixbuf).icon_name = "emblem-symbolic-link";
				}
				else if (item.file_type == FileType.DIRECTORY) {
					(cell as Gtk.CellRendererPixbuf).stock_id = "gtk-directory";
				}
				else{
					(cell as Gtk.CellRendererPixbuf).stock_id = "gtk-file";
				}
			}
			
			if (odd_row){
				(cell as Gtk.CellRendererPixbuf).cell_background = "#F4F6F7";
			}
			else{
				(cell as Gtk.CellRendererPixbuf).cell_background = "#FFFFFF";
			}

			if (App.use_large_icons_file_pane){
				(cell as Gtk.CellRendererPixbuf).ypad = 6;
				(cell as Gtk.CellRendererPixbuf).stock_size = IconSize.LARGE_TOOLBAR;
			}
			else{
				(cell as Gtk.CellRendererPixbuf).ypad = 0;
				(cell as Gtk.CellRendererPixbuf).stock_size = IconSize.MENU;
			}
		});

		// render text
		col.set_cell_data_func (cell_text, (cell_layout, cell, model, iter) => {
			FileItem item;
			bool odd_row;
			model.get (iter, 0, out item, 1, out odd_row, -1);
			
			string s = item.file_name;
			if (item.is_symlink){
				s += " → " + item.symlink_target;
				s = "<span foreground=\"#2471A3\">%s</span>".printf(s);
				(cell as Gtk.CellRendererText).markup = s;
			}
			else{
				(cell as Gtk.CellRendererText).text = s;
			}

			if (odd_row){
				(cell as Gtk.CellRendererText).background = "#F4F6F7";
			}
			else{
				(cell as Gtk.CellRendererText).background = "#FFFFFF";
			}
		});

		// sort on click
		col.clicked.connect(() => {
			if (sort_column_index == 0) {
				sort_column_desc = !sort_column_desc;
			}
			else {
				sort_column_index = 0;
				sort_column_desc = true;
			}
			tv_files_refresh();
		});

		// size -------------------------------------------------

		// column
		col = new TreeViewColumn();
		col.title = _("Size");
		col.clickable = true;
		col.resizable = true;
		col.spacing = 1;
		//col.min_width = 100;
		tv_files.append_column(col);
		col_size = col;

		// cell text
		cell_text = new CellRendererText ();
		cell_text.xalign = (float) 1.0;
		col.pack_start (cell_text, false);

		// render text
		col.set_cell_data_func (cell_text, (cell_layout, cell, model, iter) => {
			FileItem item;
			bool odd_row;
			model.get (iter, 0, out item, 1, out odd_row, -1);

			if (!item.is_dummy) {
				if (item.size > 0) {
					(cell as Gtk.CellRendererText).text = format_file_size(item.size);
				}
				else{
					(cell as Gtk.CellRendererText).text = "--";
				}
			}
			else {
				(cell as Gtk.CellRendererText).text = "";
			}

			if (odd_row){
				(cell as Gtk.CellRendererText).background = "#F4F6F7";
			}
			else{
				(cell as Gtk.CellRendererText).background = "#FFFFFF";
			}
		});

		// sort on click
		col.clicked.connect(() => {
			if (sort_column_index == 1) {
				sort_column_desc = !sort_column_desc;
			}
			else {
				sort_column_index = 1;
				sort_column_desc = true;
			}
			tv_files_refresh();
		});

		// compressed -------------------------------------------

		// column
		col = new TreeViewColumn();
		col.title = _("Packed");
		col.clickable = true;
		col.resizable = true;
		col.spacing = 1;
		//col.min_width = 100;
		tv_files.append_column(col);
		col_compressed = col;

		// cell text
		cell_text = new CellRendererText();
		cell_text.xalign = (float) 1.0;
		col.pack_start (cell_text, false);

		//render text
		col.set_cell_data_func (cell_text, (cell_layout, cell, model, iter) => {
			FileItem item;
			bool odd_row;
			model.get (iter, 0, out item, 1, out odd_row, -1);

			if (!item.is_dummy){
				if (item.size_compressed > 0) {
					(cell as Gtk.CellRendererText).text = format_file_size(item.size_compressed);
				}
				else {
					(cell as Gtk.CellRendererText).text = "--";
				}
			}
			else{
				(cell as Gtk.CellRendererText).text = "";
			}

			if (odd_row){
				(cell as Gtk.CellRendererText).background = "#F4F6F7";
			}
			else{
				(cell as Gtk.CellRendererText).background = "#FFFFFF";
			}
		});

		// sort on click
		col.clicked.connect(() => {
			if (sort_column_index == 2) {
				sort_column_desc = !sort_column_desc;
			}
			else {
				sort_column_index = 2;
				sort_column_desc = true;
			}
			tv_files_refresh();
		});

		// modified date ---------------------------------------------

		// column
		col = new TreeViewColumn();
		col.title = _("Modified");
		col.clickable = true;
		col.resizable = true;
		col.spacing = 1;
		//col.min_width = 100;
		tv_files.append_column(col);
		col_modified = col;

		// cell text
		cell_text = new CellRendererText ();
		col.pack_start (cell_text, false);

		// render text
		col.set_cell_data_func (cell_text, (cell_layout, cell, model, iter) => {
			FileItem item;
			bool odd_row;
			model.get (iter, 0, out item, 1, out odd_row, -1);

			if (!item.is_dummy){
				if (item.modified != null) {
					(cell as Gtk.CellRendererText).text = item.modified.format ("%Y-%m-%d %H:%M");
				}
				else {
					(cell as Gtk.CellRendererText).text = "--";
				}
			}
			else{
				(cell as Gtk.CellRendererText).text = "";
			}

			if (odd_row){
				(cell as Gtk.CellRendererText).background = "#F4F6F7";
			}
			else{
				(cell as Gtk.CellRendererText).background = "#FFFFFF";
			}
		});

		// sort on click
		col.clicked.connect(() => {
			if (sort_column_index == 3) {
				sort_column_desc = !sort_column_desc;
			}
			else {
				sort_column_index = 3;
				sort_column_desc = true;
			}
			tv_files_refresh();
		});

		// permissions -------------------------------------------

		// column
		col = new TreeViewColumn();
		col.title = _("Permissions");
		col.clickable = true;
		col.resizable = true;
		col.spacing = 1;
		//col.min_width = 100;
		tv_files.append_column(col);
		col_permissions = col;

		// cell text
		cell_text = new CellRendererText ();
		col.pack_start (cell_text, false);

		// render text
		col.set_cell_data_func (cell_text, (cell_layout, cell, model, iter) => {
			FileItem item;
			bool odd_row;
			model.get (iter, 0, out item, 1, out odd_row, -1);

			if (!item.is_dummy){
				if (item.permissions.length > 0) {
					(cell as Gtk.CellRendererText).text = item.permissions;
				}
				else {
					(cell as Gtk.CellRendererText).text = "--";
				}
			}
			else{
				(cell as Gtk.CellRendererText).text = "";
			}

			if (odd_row){
				(cell as Gtk.CellRendererText).background = "#F4F6F7";
			}
			else{
				(cell as Gtk.CellRendererText).background = "#FFFFFF";
			}
		});

		// sort on click
		col.clicked.connect(() => {
			if (sort_column_index == 4) {
				sort_column_desc = !sort_column_desc;
			}
			else {
				sort_column_index = 4;
				sort_column_desc = true;
			}
			tv_files_refresh();
		});

		// owner -------------------------------------------

		// column
		col = new TreeViewColumn();
		col.title = _("Owner");
		col.clickable = true;
		col.resizable = true;
		col.spacing = 1;
		//col.min_width = 100;
		tv_files.append_column(col);
		col_owner = col;
		
		// cell text
		cell_text = new CellRendererText ();
		col.pack_start (cell_text, false);

		// render text
		col.set_cell_data_func (cell_text, (cell_layout, cell, model, iter) => {
			FileItem item;
			bool odd_row;
			model.get (iter, 0, out item, 1, out odd_row, -1);

			if (!item.is_dummy){
				if (item.owner.length > 0) {
					(cell as Gtk.CellRendererText).text = item.owner;
				}
				else {
					(cell as Gtk.CellRendererText).text = "--";
				}
			}
			else{
				(cell as Gtk.CellRendererText).text = "";
			}

			if (odd_row){
				(cell as Gtk.CellRendererText).background = "#F4F6F7";
			}
			else{
				(cell as Gtk.CellRendererText).background = "#FFFFFF";
			}
		});

		// sort on click
		col.clicked.connect(() => {
			if (sort_column_index == 5) {
				sort_column_desc = !sort_column_desc;
			}
			else {
				sort_column_index = 5;
				sort_column_desc = true;
			}
			tv_files_refresh();
		});

		// group ----------------------------------------------

		// column
		col = new TreeViewColumn();
		col.title = _("Group");
		col.clickable = true;
		col.resizable = true;
		col.spacing = 1;
		//col.min_width = 100;
		tv_files.append_column(col);
		col_group = col;
		
		// cell text
		cell_text = new CellRendererText ();
		col.pack_start (cell_text, false);

		// render text
		col.set_cell_data_func (cell_text, (cell_layout, cell, model, iter) => {
			FileItem item;
			bool odd_row;
			model.get (iter, 0, out item, 1, out odd_row, -1);

			if (!item.is_dummy){
				if (item.group != null) {
					(cell as Gtk.CellRendererText).text = item.group;
				}
				else {
					(cell as Gtk.CellRendererText).text = "--";
				}
			}
			else{
				(cell as Gtk.CellRendererText).text = "";
			}

			if (odd_row){
				(cell as Gtk.CellRendererText).background = "#F4F6F7";
			}
			else{
				(cell as Gtk.CellRendererText).background = "#FFFFFF";
			}
		});

		//sort on click
		col.clicked.connect(() => {
			if (sort_column_index == 6) {
				sort_column_desc = !sort_column_desc;
			}
			else {
				sort_column_index = 6;
				sort_column_desc = true;
			}
			tv_files_refresh();
		});

		// buffer ------------------------------------------------

		col = new TreeViewColumn();
		col.title = "";
		col.clickable = false;
		col.resizable = false;
		col.min_width = 20;
		tv_files.append_column(col);

		// cell text
		cell_text = new CellRendererText ();
		col.pack_start (cell_text, false);

		//render text
		col.set_cell_data_func (cell_text, (cell_layout, cell, model, iter) => {
			FileItem item;
			bool odd_row;
			model.get (iter, 0, out item, 1, out odd_row, -1);

			(cell as Gtk.CellRendererText).text = "";

			if (odd_row){
				(cell as Gtk.CellRendererText).background = "#F4F6F7";
			}
			else{
				(cell as Gtk.CellRendererText).background = "#FFFFFF";
			}
		});
		
		//row_activated event
		tv_files.row_activated.connect(tv_files_row_activated);

		//TODO: Add flat, dual-pane and expanded views

		init_pathbar(vbox);
		
		/*// path links ----------------------------------

		hbox_path = new Box(Orientation.HORIZONTAL, 0);
		hbox_path.homogeneous = false;
		hbox_path.spacing = 0;

		pathbar_add_nav_pane_toggle();

		// add dummy widget - maintains hbox height when empty
		var dummy = new Gtk.LinkButton("");
		dummy.sensitive = false;
		hbox_path.add(dummy); 

		var scroll = new ScrolledWindow(null, null);
		scroll.set_shadow_type (ShadowType.ETCHED_IN);
		scroll.hexpand = true;
		scroll.set_size_request(-1, dummy.get_allocated_height());
		scroll.hscrollbar_policy = PolicyType.AUTOMATIC;
		scroll.vscrollbar_policy = PolicyType.NEVER;
		scroll.add (hbox_path);
		vbox.add(scroll);

		// remove linkbutton padding
		var provider = new Gtk.CssProvider();
		var css = """
		GtkLinkButton { padding-left: 3; padding-right: 3;}
		""";
		try {
			provider.load_from_data(css,-1);
		}
		catch (GLib.Error e) {
            warning(e.message);
        }
        var screen = Gdk.Screen.get_default();
        Gtk.StyleContext.add_provider_for_screen(screen,provider,STYLE_PROVIDER_PRIORITY_USER);
        * */
	}

	private void init_pathbar(Gtk.Box vbox){
		// create box
		hbox_path = new Box(Orientation.HORIZONTAL, 0);
		hbox_path.homogeneous = false;
		hbox_path.spacing = 0;

		// add scrollbars
		var scroll = new ScrolledWindow(null, null);
		scroll.set_shadow_type (ShadowType.ETCHED_IN);
		scroll.hexpand = true;
		scroll.hscrollbar_policy = PolicyType.AUTOMATIC;
		scroll.vscrollbar_policy = PolicyType.NEVER;
		scroll.add (hbox_path);
		vbox.add(scroll);

		// add nav pane toggle
		pathbar_add_nav_pane_toggle();

		// add dummy widget - maintains hbox height when empty
		var dummy = pathbar_add_spacer();
		scroll.set_size_request(-1, dummy.get_allocated_height());
		
		// remove linkbutton padding
		var provider = new Gtk.CssProvider();
		var css = """
		GtkLinkButton { padding-left: 1px; padding-right: 1px;}
		""";
		try {
			provider.load_from_data(css,-1);
		}
		catch (GLib.Error e) {
            warning(e.message);
        }
        var screen = Gdk.Screen.get_default();
        Gtk.StyleContext.add_provider_for_screen(screen,provider,STYLE_PROVIDER_PRIORITY_USER);
	}

	private void init_statusbar() {
		//hbox_statusbar
		hbox_statusbar = new Box (Orientation.HORIZONTAL, 3);
		hbox_statusbar.margin = 3;
		vbox_main.add (hbox_statusbar);

		//spinner
		spinner = new Gtk.Spinner();
		spinner.visible = spinner.active = false;
		spinner.no_show_all = true;
		hbox_statusbar.add(spinner);

		//lbl_statusbar
		lbl_statusbar = new Label ("");
		lbl_statusbar.xalign = (float) 0.0;
		lbl_statusbar.hexpand = true;
		hbox_statusbar.add(lbl_statusbar);

		//progressbar
		progressbar = new Gtk.ProgressBar();
		progressbar.pulse_step = 0.2;
		progressbar.no_show_all = true;
		hbox_statusbar.add(progressbar);
	}

	private void btn_about_clicked () {
		var dialog = new AboutWindow();
		dialog.set_transient_for (this);

		dialog.authors = {
			"Tony George:teejeetech@gmail.com"
		};

		dialog.translators = {
			//"giulux (Italian)",
			//"Jorge Jamhour (Brazilian Portuguese):https://launchpad.net/~jorge-jamhour",
			//"B. W. Knight (Korean):https://launchpad.net/~kbd0651",
			//"Rodion R. (Russian):https://launchpad.net/~r0di0n"
		};

		dialog.documenters = null;
		dialog.artists = null;
		dialog.donations = null;

		dialog.program_name = AppName;
		dialog.comments = _("A modern, light-weight archive manager for Linux");
		dialog.copyright = "Copyright © 2016 Tony George (%s)".printf(AppAuthorEmail);
		dialog.version = AppVersion;
		dialog.logo = get_app_icon(128,".svg");

		dialog.license = "This program is free for personal and commercial use and comes with absolutely no warranty. You use this program entirely at your own risk. The author will not be liable for any damages arising from the use of this program.";
		dialog.website = "http://teejeetech.in";
		dialog.website_label = "http://teejeetech.blogspot.in";

		dialog.initialize();
		dialog.show_all();
	}

	private void ui_set_active(bool is_active){
		toolbar.sensitive = is_active;
		tv_nav.sensitive = is_active;
		tv_files.sensitive = is_active;
		hbox_path.sensitive = is_active;
		gtk_set_busy(!is_active, this);
		gtk_do_events();
	}

	private void first_run(){
		if (App.first_run){
			App.first_run = false;
			App.save_app_config();
			
			foreach(var mime in App.mimetype_list) {
				MimeApp.set_default(mime.type, App.crunchy_app);
			}
			MimeApp.query_mimeapps(App.user_home);

			App.update_context_menu();

			string s = "";

			s += "1. " + _("Crunchy has been set as the default archive manager. You can change this from the <b>Settings</b> window.") + "\n\n";
			
			s += "2. " + _("Recommended archive format for best compression is <b>TAR + XZ</b>. This uses TAR for archiving and LZMA2 for compression.") + "\n\n";

			s += "3. " + _("<b>TAR</b> is an extremely versatile file format which is well suited for Linux file systems. It can preserve unix-specific file metadata such as permissions, owner, group, etc. It also knows how to handle symlinks, hard-links and sparse files (symlinks are stored as symlinks and hard-linked files are stored only once)") + "\n\n";

			s += "4. " + _("<b>LZMA2</b> is an improved version of the LZMA compression method used by 7zip. It can compress as well as LZMA while being faster due to better multi-threading. It is better at handling un-compressible data (audio, video, etc). Such data is stored without compression instead of trying to compress it further.") + "\n\n";

			s += "5. " + _("Formats such as <b>7Z and ZIP</b> should not be used on Linux as these are designed for Windows. They cannot store file permissions, owner and group information. Use these formats only if you are okay with losing this information, or for sharing files with Windows users.") + "\n\n";

			gtk_messagebox("First Run",s, this, false, true);
		}
	}
	
	//refresh

	/*private void refesh_view_title_and_summary() {
		current_item = App.archive;
		tv_nav_refresh();
		tv_files_refresh();
		set_statusbar_summary();
		set_window_title();
		set_toolbar_buttons();
	}*/

	private void tv_nav_refresh() {
		log_debug("tv_nav_refresh()");

		TreeIter iter0;
		TreeStore model = new TreeStore (2, typeof(FileItem), typeof(bool));
		
		//fix for compile error for unassigned iter0
		model.append (out iter0, null);
		model.clear();

		bool odd_row = true;
		foreach(var arch in archive_list){
			model.append (out iter0, null);
			model.set (iter0, 0, arch);
			model.set (iter0, 1, odd_row);
			odd_row = !odd_row;
			
			tv_nav_append_to_iter(ref model, ref iter0, arch);
		}

		tv_nav.set_model(model);
		tv_nav.columns_autosize();

		if (tv_nav.get_selection().count_selected_rows() > 0){
			tv_nav.get_selection().select_iter(iter0);
		}

		if (App.hide_nav_pane){
			pane.position = 0;
		}
		else{
			if (pane.position < 100){
				pane.position = (int) (this.get_allocated_width() / 4);
			}
		}
	}
	
	private void tv_nav_refresh_selected() {
		log_debug("tv_nav_refresh_selected()");
		
		TreeIter iter0, iter1;
		TreeModel model;
		tv_nav.get_selection().get_selected(out model, out iter0);

		FileItem item;
		tv_nav.model.get (iter0, 0, out item, -1);

		while (model.iter_children(out iter1, iter0)) {
			((TreeStore)model).remove(ref iter1);
		}

		var store = (TreeStore) model;
		tv_nav_append_to_iter(ref store, ref iter0, current_item, false);
		tv_nav.expand_row(model.get_path(iter0), false);
	}

	private void tv_nav_row_activated(TreePath path, TreeViewColumn column){
		log_debug("tv_nav_row_activated()");
		
		TreeIter iter;
		tv_nav.model.get_iter_from_string(out iter, path.to_string());
		FileItem item;
		tv_nav.model.get (iter, 0, out item, -1);
		
		current_task = (ArchiveFile) item.source_archive;
		current_item = item;
		
		App.archive = current_task;

		// check if close icon was clicked -----------
		
		if ((item.parent == null) && (column == col_close)){
			archive_list.remove(current_task);
			current_task = null;
			current_item = null;
			
			App.archive.remove_temp_files();
			App.archive = null;
			
			tv_nav_refresh();
		}

		// update file list ------------------------
		
		set_window_title();
		set_toolbar_buttons();

		tv_files_refresh();
		set_statusbar_summary();
		pathbar_update();
	}
	
	private void tv_nav_select_child_from_file_view() {
		log_debug("tv_nav_select_child_from_file_view()");
		
		// select and expand a directory from nav pane,
		// when the directory is double-clicked in file view
		
		TreeIter iter0;
		TreeModel model;

		tv_nav.get_selection().get_selected(out model, out iter0);
		tv_nav.expand_row(tv_nav.model.get_path(iter0), false);

		TreeIter iter1, iter_selected = iter0;
		bool found = false;
		bool has_children = model.iter_children(out iter1, iter0);

		do {
			if (!has_children) {
				break;
			}

			FileItem child;
			tv_nav.model.get (iter1, 0, out child, -1);
			if (child.file_path == current_item.file_path) {
				iter_selected = iter1;
				found = true;
				break;
			}
		}
		while (model.iter_next(ref iter1));

		if (found) {
			tv_nav.get_selection().select_iter(iter_selected);
			tv_nav.expand_row(model.get_path(iter_selected), false);
		}
	}

	private void tv_nav_select_parent_from_file_view() {
		log_debug("tv_nav_select_parent_from_file_view()");
		
		// select and collapse the parent directory from nav pane,
		// when the parent directory entry (..) is double-clicked in file view
		
		TreeIter iter0;
		TreeModel model;

		tv_nav.get_selection().get_selected(out model, out iter0);

		TreeIter iter1;
		if (model.iter_parent(out iter1, iter0)) {
			tv_nav.get_selection().select_iter(iter1);
			tv_nav.expand_row(model.get_path(iter1), false);
		}
	}

	private TreeIter tv_nav_append_to_iter(ref TreeStore model, ref TreeIter iter0, FileItem item, bool addItem = true) {
		//append sub-directories to the nav pane iter
		
		TreeIter iter1 = iter0;
		if (addItem && (item.parent != null)) {
			model.append (out iter1, iter0);
			model.set (iter1, 0, item);
		}

		var list = new ArrayList<FileItem>();
		foreach(string key in item.children.keys) {
			var child = item.children[key];
			if ((child.file_type == FileType.DIRECTORY) && !child.is_symlink) {
				list.add(child);
			}
		}

		list.sort((a, b) => {
			return strcmp(a.file_name, b.file_name);
		});

		foreach(var child in list) {
			tv_nav_append_to_iter(ref model, ref iter1, child);
		}

		return iter1;
	}


	private void tv_files_refresh() {
		log_debug("tv_files_refresh()");

		var model = new Gtk.TreeStore(2, typeof(FileItem), typeof(bool));
		TreeIter iter;

		if (current_item == null){
			tv_files.set_model(model);
			tv_files.columns_autosize();
			return;
		}
		
		var list = new ArrayList<FileItem>();
		foreach(string key in current_item.children.keys) {
			var item = current_item.children[key];
			list.add(item);
		}

		switch (sort_column_index) {
		case 0:
			list.sort((a, b) => {
				if (a.file_type != b.file_type) {
					if (a.file_type == FileType.DIRECTORY) {
						return -1;
					}
					else {
						return +1;
					}
				}
				else {
					if (sort_column_desc) {
						return strcmp(a.file_name, b.file_name);
					}
					else {
						return -1 * strcmp(a.file_name, b.file_name);
					}
				}
			});
			break;

		case 1:
			list.sort((a, b) => {
				if (sort_column_desc) {
					return ((int)(a.size - b.size));
				}
				else {
					return -1 * ((int)(a.size - b.size));
				}
			});
			break;

		case 2:
			list.sort((a, b) => {
				if (sort_column_desc) {
					return ((int)(a.size_compressed - b.size_compressed));
				}
				else {
					return -1 * ((int)(a.size_compressed - b.size_compressed));
				}
			});
			break;

		case 3:
			list.sort((a, b) => {
				if (sort_column_desc) {
					return a.modified.compare(b.modified);
				}
				else {
					return -1 * a.modified.compare(b.modified);
				}
			});
			break;

		case 4:
			list.sort((a, b) => {
				if (sort_column_desc) {
					return strcmp(a.permissions, b.permissions);
				}
				else {
					return -1 * strcmp(a.permissions, b.permissions);
				}
			});
			break;
		case 5:
			list.sort((a, b) => {
				if (sort_column_desc) {
					return strcmp(a.owner, b.owner);
				}
				else {
					return -1 * strcmp(a.owner, b.owner);
				}
			});
			break;
		case 6:
			list.sort((a, b) => {
				if (sort_column_desc) {
					return strcmp(a.group, b.group);
				}
				else {
					return -1 * strcmp(a.group, b.group);
				}
			});
			break;
		}

		if (current_item.parent != null) {
			var dummy = new FileItem.dummy(FileType.DIRECTORY);
			dummy.file_name = "..";
			dummy.parent = current_item.parent;

			//add row for parent dir
			model.append(out iter, null);
			model.set (iter, 0, dummy);
		}

		bool first_iter_found = false;
		TreeIter first_iter;
		bool odd_row = false;
		foreach(FileItem item in list) {
			odd_row = !odd_row;
			
			//add row
			model.append(out iter, null);
			model.set (iter, 0, item, 1, odd_row);

			if (!first_iter_found) {
				first_iter_found = true;
				first_iter = iter;
			}
		}

		tv_files.set_model(model);
		tv_files.columns_autosize();

		if (App.archive.file_path.length > 0) {
			switch (App.archiver.archiver_name) {
			case "tar":
				col_permissions.visible = col_owner.visible = col_group.visible = true;
				col_compressed.visible = false;
				break;
			case "7z":
				col_permissions.visible = col_owner.visible = col_group.visible = false;
				col_compressed.visible = !App.archive.archive_is_solid;
				break;
			}
		}
		else {
			col_permissions.visible = col_owner.visible = col_group.visible = false;
			col_compressed.visible = false;
		}

	}

	private void tv_files_row_activated(TreePath path, TreeViewColumn column){
		log_debug("tv_files_row_activated()");
		
		TreeIter iter;
		Gtk.TreeStore model = (Gtk.TreeStore) tv_files.model;
		model.get_iter_from_string(out iter, path.to_string());
		FileItem item;
		model.get (iter, 0, out item, -1);

		change_current_item(item);
	}

	// path bar ----------------------------------------
	
	private void pathbar_update(){
		// remove existing
		foreach(var child in hbox_path.get_children()){
			hbox_path.remove(child);
		}

		pathbar_add_nav_pane_toggle();

		pathbar_add_root_link();

		pathbar_add_path_links();
		
		hbox_path.show_all();
	}

	private void pathbar_add_nav_pane_toggle(){
		Gtk.Image img = null;
		if (App.hide_nav_pane){
			img = get_shared_icon("","pane-show.svg",16);
			img.set_tooltip_text(_("Show Navigation Pane"));
		}
		else{
			img = get_shared_icon("","pane-hide.svg",16);
			img.set_tooltip_text(_("Hide Navigation Pane"));
		}
		img.margin_left = 6;
		img.margin_right = 6;
		img_nav_toggle = img;
		
		var ebox = new Gtk.EventBox();
		ebox.add(img_nav_toggle);
		hbox_path.add(ebox);

		// set hand cursor
		if (ebox.get_realized()){
			ebox.get_window().set_cursor(new Gdk.Cursor(Gdk.CursorType.HAND1));
		}
		else{
			ebox.realize.connect(()=>{
				ebox.get_window().set_cursor(new Gdk.Cursor(Gdk.CursorType.HAND1));
			});
		}

		ebox.button_press_event.connect((event)=>{
			App.hide_nav_pane = !App.hide_nav_pane;

			if (App.hide_nav_pane){
				var img_file = "/usr/share/%s/images/%s".printf(AppShortName, "pane-show.svg");
				img_nav_toggle.set_from_file(img_file);
				img_nav_toggle.set_tooltip_text(_("Show Navigation Pane"));
			}
			else{
				var img_file = "/usr/share/%s/images/%s".printf(AppShortName, "pane-hide.svg");
				img_nav_toggle.set_from_file(img_file);
				img_nav_toggle.set_tooltip_text(_("Hide Navigation Pane"));
			}
			tv_nav_refresh();
			return true;
		});	
	}

	private void pathbar_add_root_link(){

		if (current_task == null){
			return;
		}
		
		var img = get_shared_icon("archive","archive.svg",16);
		img.margin_right = 6;
		img.set_tooltip_text(_("Archive Root"));

		var ebox = new Gtk.EventBox();
		ebox.add(img);
		hbox_path.add(ebox);

		// set hand cursor
		if (ebox.get_realized()){
			ebox.get_window().set_cursor(new Gdk.Cursor(Gdk.CursorType.HAND1));
		}
		else{
			ebox.realize.connect(()=>{
				ebox.get_window().set_cursor(new Gdk.Cursor(Gdk.CursorType.HAND1));
			});
		}

		ebox.button_press_event.connect((event)=>{
			if (current_task != null){
				current_item = current_task;
				tv_nav_select_child_from_file_view();
				tv_files_refresh();
				set_statusbar_summary();
				pathbar_update();
				return true;
			}
			else{
				log_error("Current task is null");
				return false;
			}
		});

		hbox_path.add(new Gtk.Label("/"));
	}

	private void pathbar_add_path_links(){

		if (current_item == null){
			pathbar_add_spacer();
			return;
		}

		if (current_item == current_task){
			pathbar_add_spacer();
			return;
		}
		
		// get full path
		string link_path = "/";
		string full_path = GLib.Path.build_filename(
			current_item.file_path_prefix, current_item.file_path);

		// check if empty (base path)
		if (full_path.length == 0){
			// add dummy widget - maintains hbox height when empty
			var dummy = new Gtk.LinkButton("");
			dummy.sensitive = false;
			hbox_path.add(dummy);
			return;
		}
		
		foreach(var part in full_path.split("/")){
			if (part.length == 0){
				continue;
			}
			
			link_path = GLib.Path.build_filename(link_path, part);
			
			var link = new Gtk.LinkButton(part);
			link.margin = 0;
			link.uri = link_path;
			link.activate_link.connect(() => {
				if (current_task != null){
					current_item = current_task.find_descendant(link.uri);
					tv_nav_select_child_from_file_view();
					tv_files_refresh();
					set_statusbar_summary();
					pathbar_update();
					return true;
				}
				else{
					log_error("Current task is null");
					return false;
				}
			});

			hbox_path.add(link);
			hbox_path.add(new Gtk.Label("/"));
		}

		pathbar_add_spacer();
	}

	private Gtk.LinkButton pathbar_add_spacer(){
		// add spacer
		var dummy = new Gtk.LinkButton("");
		dummy.sensitive = false;
		hbox_path.add(dummy);
		return dummy;
	}

	private void extract_and_list(FileItem item){
		
		// save task ref
		var task_bak = App.archive;

		// disable ui
		ui_set_active(false);
		spinner.visible = spinner.active = true;
		open_archive_task_is_running = true;
		
		// get the archive from file item's tag (may be a nested archive)
		App.archive = (ArchiveFile) item.source_archive;
		App.archive.extract_list.add(item.file_path);
		App.archiver.extraction_path = App.archive.temp_dir;
		
		log_debug("Extracting: %s".printf(item.file_path));
		task_is_running = true;
		gtk_do_events();

		App.extract();
		gtk_do_events();

		while (App.archiver.status == AppStatus.RUNNING){
			sleep(200);
			gtk_do_events();
		}
		
		var extracted_file = "%s/%s".printf(App.archive.temp_dir, item.file_path);
		if (file_exists(extracted_file)){

			// create task for listing
			App.archive = new ArchiveFile(extracted_file);

			// list contents
			App.open();
			while (App.archiver.status == AppStatus.RUNNING){
				sleep(200);
				gtk_do_events();
			}
			
			log_debug("Listing done");

			// assign children and parents
			item.children = App.archive.children;
			foreach(var fi in item.children.values){
				fi.parent = item;
				fi.set_file_path_prefix(item.file_path);
			}

			// mark as archive
			item.is_archive = true;
		}
		else{
			log_error(_("File not found") + ": %s".printf(extracted_file));
		}

		// enable ui
		ui_set_active(true);
		spinner.visible = spinner.active = false;
		open_archive_task_is_running = false;
		set_statusbar_summary();

		// restore reference
		App.archive = task_bak;
	}
	
	private void set_window_title() {
		log_debug("set_window_title()");
		
		if (current_task != null){
			this.title = file_basename(App.archive.file_path);
		}
		else {
			this.title = AppName + " v" + AppVersion;
		}
	}

	private void set_statusbar_summary(string prefix = "") {
		//log_debug("set_statusbar_summary()");
		
		//lbl_statusbar.label = "%ld files, %ld dirs, %s size".printf(App.archive.file_count_total,App.archive.dir_count_total,format_file_size(App.archive.byte_count_total));
		
		
		if (statusbar_msg.length > 0) {
			lbl_statusbar.label = statusbar_msg;
		}
		else if (App.archive == null){
			lbl_statusbar.label = "";
		}
		else if (open_archive_task_is_running){
			lbl_statusbar.label = App.status_line;
		}
		else {
			//bool new_archive = (App.archive.file_path.length == 0);
			//bool no_files_in_archive = (App.archive.children.size == 0);

			var txt = "";
			if (current_task == null) {
				//txt += "Add the files to compress and click the 'Save' button on the toolbar";
			}
			else {
				txt += "Files: %'ld | Dirs: %'ld | Size: %s".printf(current_item.file_count_total, current_item.dir_count_total, format_file_size(current_item.size));
				//we will show 0 counts if archive is empty
			}

			if (App.archive.file_path.length > 0) {
				txt += " | Packed: %s".printf(format_file_size(App.archive.archive_size));
			}

			if (App.archive.archive_is_encrypted) {
				txt += " | Encrypted";
			}
			
			if (App.archive.archive_is_solid) {
				txt += " | Solid";
			}

			if (prefix.length > 0) {
				txt = prefix + txt;
			}
			lbl_statusbar.label = txt;
		}
	}

	private void set_toolbar_buttons(){
		log_debug("set_toolbar_buttons()");
		
		if ((archive_list.size == 0) || (current_task == null)
			|| (current_task.file_path.length == 0)){
				
			//new archive
			//btn_create_archive.visible = btn_add_files.visible = btn_add_folders.visible = btn_remove_item.visible = true;
			btn_properties.visible = btn_extract_archive.visible = btn_test_archive.visible = false;
		}
		else{
			//existing archive
			//btn_create_archive.visible = btn_add_files.visible = btn_add_folders.visible = btn_remove_item.visible = false;
			btn_properties.visible = btn_extract_archive.visible = btn_test_archive.visible = true;
		}
	}

	private void change_current_item(FileItem item){

		// check and list archive files ---------
		
		if (item.is_archive && (item.children.size == 0)){
			extract_and_list(item);
		}

		// select item ------------
		
		if (((item.file_type == FileType.DIRECTORY) && (!item.is_symlink)) || item.is_archive){
			
			if (item.is_dummy && (item.parent != null)) {
				current_item = item.parent;
				tv_nav_select_parent_from_file_view();
			}
			else {
				current_item = item;
				tv_nav_select_child_from_file_view();
			}

			tv_files_refresh();
			set_statusbar_summary();
		}

		// update location links -------------
		
		pathbar_update();
	}
	
	// open -----------------------------
	
	private void open_archive_select_file() {
		log_debug("open_archive_select_file()");
		
		//chooser
		var chooser = new Gtk.FileChooserDialog(
		    "Select the archive to open",
		    this,
		    FileChooserAction.OPEN,
		    "_Cancel",
		    Gtk.ResponseType.CANCEL,
		    "_Open",
		    Gtk.ResponseType.ACCEPT
		);

		chooser.select_multiple = true;

		chooser.add_filter(create_file_filter("All Files", { "*" }));
		chooser.add_filter(create_file_filter("7-Zip archive (*.7z, *.tar.7z)", { "*.7z", "*.tar.7z" }));
		chooser.add_filter(create_file_filter("Apple disk image (*.dmg)",  { "*.dmg" }));
		chooser.add_filter(create_file_filter("ARJ archive (*.arj)",  { "*.arj" }));
		chooser.add_filter(create_file_filter("BZip2 archive (*.bzip2, *.bz2, *.tar.bzip2, *.tar.bz2, *.tbz2, *.tbz, *.tb2)", { "*.bzip2", "*.bz2", "*.tar.bzip2", "*.tar.bz2", "*.tbz2", "*.tbz", "*.tb2" }));
		chooser.add_filter(create_file_filter("Cabinet archive (*.cab)",  { "*.cab" }));
		chooser.add_filter(create_file_filter("CPIO archive (*.cpio)",  { "*.cpio" }));
		chooser.add_filter(create_file_filter("Compressed help file (*.chm, *.chw, *.hxs)",  { "*.chm", "*.chw", "*.hxs" }));
		chooser.add_filter(create_file_filter("CD-DVD image (*.iso)",  { "*.iso" }));
		chooser.add_filter(create_file_filter("Debian package (*.deb)",  { "*.deb" }));
		chooser.add_filter(create_file_filter("Disk image (*.fat, *.ntfs, *.hfs, *.squashfs, *.cramfs, *.mbr)",  { "*.ntfs", "*.fat", "*.hfs", "*.squashfs", "*.cramfs", "*.mbr" }));
		chooser.add_filter(create_file_filter("EFI firmware file (*.scap)",  { "*.scap" }));
		chooser.add_filter(create_file_filter("GZip archive (*.gz, *.gzip, *.tar.gz, *.tgz)",  { "*.gz", "*.gzip", "*.tar.gz", "*.tgz" }));
		chooser.add_filter(create_file_filter("LZH archive (*.lzh, *.lha)",  { "*.lzh", "*.lha" }));
		chooser.add_filter(create_file_filter("LZMA archive (*.lzma,*.tar.lzma,*.tar.lz,*.tlz)",  { "*.lzma", "*.tar.lzma", "*.tar.lz", "*.tlz" }));
		chooser.add_filter(create_file_filter("NSIS installer (*.exe)",  { "*.exe" }));
		chooser.add_filter(create_file_filter("RAR archive (*.rar)",  { "*.rar" }));
		chooser.add_filter(create_file_filter("RedHat package (*.rpm)",  { "*.rpm" }));
		chooser.add_filter(create_file_filter("TAR archive (*.tar)",   { "*.tar" }));
		chooser.add_filter(create_file_filter("Virtual hard disk image (*.vhd)",   { "*.vhd" }));
		chooser.add_filter(create_file_filter("Windows imaging format (*.wim, *.swm)",   { "*.wim", "*.swm" }));
		chooser.add_filter(create_file_filter("XAR archive (*.xar)",   { "*.xar" }));
		chooser.add_filter(create_file_filter("XZ archive (*.xz, *.tar.xz, *.txz)",    { "*.xz", "*.tar.xz", "*.txz" }));
		chooser.add_filter(create_file_filter("Zip archives (*.zip,*.jar,*.war,*.ear)",   { "*.zip","*.jar","*.war","*.ear" }));
		chooser.add_filter(create_file_filter("Z archive (*.z, *.tar.z, *.taz)",  { "*.z", "*.tar.z", "*.taz" }));

		if (App.last_input_dir.length > 0){
			chooser.set_current_folder(App.last_input_dir);
		}
		else{
			chooser.set_current_folder(App.user_home);
		}
		
		if (chooser.run() == Gtk.ResponseType.ACCEPT) {

			selected_archives.clear();
			foreach(string file_path in chooser.get_filenames()){
				if (file_exists(file_path)){ // this excludes files which are broken links
					log_debug("selected: %s".printf(file_path));
					selected_archives.add(file_path);
				}
			}

			chooser.close();
			chooser.destroy();
			gtk_do_events();

			open_archives();
		}
		else {
			chooser.close();
			chooser.destroy();
			gtk_do_events();
			return;
		}
	}

	private void open_archives(){
		log_debug("open_archives()");

		ui_set_active(false);
		spinner.visible = spinner.active = true;
		open_archive_task_is_running = true; // used for customizing statusbar message
		
		foreach(string file_path in selected_archives){
			// create task
			App.archive = new ArchiveFile(file_path);
			App.archiver.action = ArchiveAction.LIST;

			current_task = App.archive;
			current_item = App.archive;
			
			App.last_input_dir = file_parent(file_path);

			open_archive();
			gtk_do_events();
			
			while (task_is_running){
				sleep(200);
				gtk_do_events();
			}
		}

		ui_set_active(true);
		spinner.visible = spinner.active = false;
		open_archive_task_is_running = false;
		set_statusbar_summary();
	}

	private void open_archive() {
		log_debug("open_archive(): %s".printf(App.archive.file_path));
		
		task_is_running = true;

		//var status_msg = _("Reading archive...");
		//dlg_open = new SimpleProgressWindow.with_parent(this, status_msg);
		//dlg_open.title = file_basename(App.archive.file_path);

		App.open();
		
		gtk_do_events();
		
		timerID = Timeout.add (500, update_status);
	}

	private void open_archive_finish(){
		log_debug("open_archive_finish()");
		
		Source.remove (timerID);

		//dlg_open.close();
		//dlg_open.destroy();

		var status = App.archiver.read_status();
		if (App.archive.archive_is_encrypted && (status != 0)){
			tmr_task = Timeout.add(200, prompt_for_password_and_open);
			// leave task_is_running=true as task is still running
		}
		else if (App.archive.children.keys.size == 0) {
			var title = "";//file_basename(App.archive.archive_path);
			var msg = _("Format is not supported");
			msg += "\n\n> " + file_basename(App.archive.file_path) + "\n";
			gtk_messagebox(title, msg, this, true);
			task_is_running = false;
		}
		else if (status == 0){

			for(int i=0; i < archive_list.size; i++){
				var archive = archive_list[i];
				if (archive.file_path == App.archive.file_path){
					archive_list.remove(archive); //remove if existing
					break;
				}
			}
			
			var archive = App.archive;
			archive_list.add(archive);

			current_task = archive;
			current_item = archive;
			App.archive = archive;
			
			tv_nav_refresh();
			tv_files_refresh();
			set_statusbar_summary();
			set_window_title();
			set_toolbar_buttons();
			pathbar_update();
			
			task_is_running = false;
		}

		//ui_set_active(true);
		//spinner.visible = spinner.active = false;
	}

	private bool prompt_for_password_and_open(){
		log_debug("prompt_for_password_and_open()");
		
		if (tmr_task > 0) {
			Source.remove(tmr_task);
			tmr_task = 0;
		}

		string txt = _("Encrypted Archive");
		string msg = _("Enter password");
		App.archive.password = PasswordDialog.prompt_user(this, false, txt, msg);

		if (App.archive.password.length > 0){
			open_archive();
		}
		else{
			task_is_running = false; // password was not provided
		}
		
		return true;
	}
	
	// extract ---------------------
	
	private void extract_archive_select_path(){
		log_debug("extract_archive_select_path()");
		
		//chooser
		var chooser = new Gtk.FileChooserDialog(
			_("Select Extraction Location"),
			this,
			FileChooserAction.SELECT_FOLDER,
			"_Cancel",
			Gtk.ResponseType.CANCEL,
			"_Open",
			Gtk.ResponseType.ACCEPT
		);

		chooser.select_multiple = false;
		//chooser.set_filename(archive_location);

		if (App.last_output_dir.length > 0){
			chooser.set_current_folder(App.last_output_dir);
		}
		else{
			chooser.set_current_folder(App.user_home);
		}

		if (chooser.run() == Gtk.ResponseType.ACCEPT) {
			App.archiver.extraction_path = chooser.get_filename();

			App.last_output_dir = App.archiver.extraction_path;
			
			chooser.close();
			extract_archive();
		}
		else{
			chooser.close();
		}
	}

	private void extract_archive() {
		log_debug("extract_archive()");
		
		toolbar.sensitive = false;
		tv_files.sensitive = false;

		gtk_do_events();

		App.archiver.action = ArchiveAction.EXTRACT;
		
		var window = new ProgressWindow.with_parent(this);
		window.destroy.connect(()=>{
			toolbar.sensitive = true;
			tv_files.sensitive = true;
		});
	}

	private void test_archive() {
		log_debug("test_archive()");
		
		toolbar.sensitive = false;
		tv_files.sensitive = false;

		gtk_do_events();

		App.archiver.action = ArchiveAction.TEST;
		
		var window = new ProgressWindow.with_parent(this);
		window.destroy.connect(()=>{
			toolbar.sensitive = true;
			tv_files.sensitive = true;
		});
	}
	
	private bool update_status() {
		switch (App.archiver.status) {
		case AppStatus.RUNNING:
			//log_debug("update_status():RUNNING");
			
			App.status_line = _("Reading archive...") + " %'ld files (%s)".printf(
								App.archive.file_count_total,
								format_file_size(App.archive.size));
								
			//dlg_open.update_message(App.status_line);


			App.progress_count = App.archive.size;

			log_debug("read: %'ld / %'ld".printf(
				App.archive.size,App.archive.archive_unpacked_size));
			
			//dlg_open.update_progressbar();
			set_statusbar_summary();
			
			break;

		case AppStatus.PASSWORD_REQUIRED:
			log_debug("update_status():PASSWORD_REQUIRED");
			
			// remove progress timers
			Source.remove (timerID);

			// close progress dialog
			dlg_open.close();
			dlg_open.destroy();

			// prompt for password
			tmr_task = Timeout.add(200, prompt_for_password_and_open);
			break;
			
		default:
			log_debug("update_status():FINISHED");
			
			//done
			open_archive_finish();
			
			break;
		}
		return true;
	}

	// new archive actions ----------------------
	
	private void add_files_to_new_archive() {
		//chooser
		var chooser = new Gtk.FileChooserDialog(
		    "Select files to add",
		    this,
		    FileChooserAction.OPEN,
		    "_Cancel",
		    Gtk.ResponseType.CANCEL,
		    "_Open",
		    Gtk.ResponseType.ACCEPT
		);

		chooser.select_multiple = true;
		//chooser.set_filename(archive_location);

		if (chooser.run() == Gtk.ResponseType.ACCEPT) {
			foreach(string uri in chooser.get_uris()) {
				current_item.add_child_from_disk(uri.replace("file://", ""));
			}
		}

		chooser.close();

		tv_nav_refresh_selected();
		tv_files_refresh();

		statusbar_msg = "";
		set_statusbar_summary();
	}

	private void add_folders_to_new_archive() {
		//chooser
		var chooser = new Gtk.FileChooserDialog(
		    "Select directories to add",
		    this,
		    FileChooserAction.SELECT_FOLDER ,
		    "_Cancel",
		    Gtk.ResponseType.CANCEL,
		    "_Open",
		    Gtk.ResponseType.ACCEPT
		);

		chooser.select_multiple = true;
		chooser.create_folders = false;

		if (chooser.run() == Gtk.ResponseType.ACCEPT) {
			foreach(string uri in chooser.get_uris()) {
				current_item.add_child_from_disk(uri.replace("file://", ""));
			}
		}

		chooser.close();

		tv_nav_refresh_selected();
		tv_files_refresh();

		statusbar_msg = "";
		set_statusbar_summary();
	}

	private void remove_selected_items() {
		var list = new Gee.ArrayList<FileItem>();
		TreeSelection sel = tv_files.get_selection();

		TreeIter iter;
		bool iterExists = tv_files.model.get_iter_first (out iter);
		while (iterExists) {
			if (sel.iter_is_selected (iter)) {
				FileItem item;
				tv_files.model.get (iter, 0, out item, -1);
				list.add(item);
			}
			iterExists = tv_files.model.iter_next (ref iter);
		}

		foreach(FileItem item in list) {
			current_item.remove_child(item.file_name);
		}

		tv_nav_refresh_selected();
		tv_files_refresh();

		set_statusbar_summary();
	}
}




